/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop;

import java.util.LinkedHashMap;
import org.jboss.aop.advice.AdviceBinding;
import org.jboss.aop.metadata.ClassMetaDataBinding;

/**
 * comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 */
public class InstanceDomain extends Domain
{
   protected Advisor advisor;

   public InstanceDomain(AspectManager manager, boolean parentFirst)
   {
      super(manager, parentFirst);
   }

   public Advisor getAdvisor()
   {
      return advisor;
   }

   public void setAdvisor(Advisor advisor)
   {
      this.advisor = advisor;
   }

   public synchronized void addBinding(AdviceBinding binding)
   {
      removeBinding(binding.getName());
      synchronized (bindings)
      {
         bindings.put(binding.getName(), binding);
      }
      if (advisor != null) advisor.newBindingAdded();
   }

   public void addClassMetaData(ClassMetaDataBinding meta)
   {
      removeClassMetaData(meta.getName());
      if (advisor != null)
      {
         if (meta.matches(advisor, advisor.getClazz()))
         {
            meta.addAdvisor(advisor);
         }
      }
   }

   public LinkedHashMap getBindings()
   {
      LinkedHashMap map = new LinkedHashMap(parent.getBindings());
      map.putAll(this.bindings);
      return map;
   }

   /**
    * internal to jboss aop.  Do not call
    * <p/>
    * This is overriden so that AdviceBinding can clear an advisor that is created per-instance.
    *
    * @param advisor
    * @return
    */
   public boolean isAdvisorRegistered(Advisor advisor)
   {
      return advisor == advisor;
   }
}
