/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.aop.pointcut;

import java.io.StringReader;

import javassist.CtClass;
import javassist.NotFoundException;
import javassist.expr.MethodCall;
import javassist.expr.NewExpr;

import org.jboss.aop.Advisor;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.ParseException;
import org.jboss.aop.pointcut.ast.PointcutExpressionParser;
import org.jboss.aop.pointcut.ast.TypeExpressionParser;

/**
 *
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.1.8.2 $
 */
public class DeclareDef
{
   String name;
   String expr;
   boolean warning;
   String msg;
   ASTStart ast;
   boolean pointcut;
   
   public DeclareDef(String name, String expr, boolean warning, String msg) throws ParseException
   {
      this.name = name;
      this.expr = expr;
      this.warning = warning;
      this.msg = msg;
      
      try
      {
         ast = new PointcutExpressionParser(new StringReader(expr)).Start();
         pointcut = true;
      } 
      catch (ParseException pe)
      {
         try
         {
            ast = new TypeExpressionParser(new StringReader(expr)).Start();
         }
         catch (ParseException te)
         {
            StringBuffer sb = new StringBuffer("The expression '" + 
                  expr + "' resolves to neither a pointcut nor a type expression");
            sb.append("\n\nPointcut parse error:\n");
            sb.append(pe.getMessage());
            sb.append("\n\nType expression parse error:\n");
            sb.append(te.getMessage());
            throw new ParseException(sb.toString());
         }
      }
   }
   

   public ASTStart getAst()
   {
      return ast;
   }
   
   public String getExpr()
   {
      return expr;
   }
   
   public String getName()
   {
      return name;
   }
   
   public boolean getWarning()
   {
      return warning;
   }
   
   public String getMsg()
   {
      return msg;
   }
   
   /**
    * @return true if a pointcut expression, false if a type expression
    */
   public boolean isPointcut()
   {
      return pointcut;
   }
   
   public boolean matches(Advisor advisor, CtClass clazz)
   {
      if (pointcut)return false;
      DeclareTypeMatcher matcher = new DeclareTypeMatcher(advisor, clazz);
      return ((Boolean) ast.jjtAccept(matcher, null)).booleanValue();
   }

   public boolean matches(Advisor advisor, Class clazz)
   {
      if (pointcut)return false;
      DeclareTypeMatcher matcher = new DeclareTypeMatcher(advisor, clazz);
      return ((Boolean) ast.jjtAccept(matcher, null)).booleanValue();
   }

   public boolean matchesCall(Advisor callingAdvisor, MethodCall methodCall) throws NotFoundException
   {
      if (!pointcut)return false;
      MethodCallMatcher matcher = new MethodCallMatcher(callingAdvisor, methodCall, ast);
      return matcher.matches();
   }

   public boolean matchesCall(Advisor callingAdvisor, NewExpr methodCall) throws NotFoundException
   {
      if (!pointcut)return false;
      NewExprMatcher matcher = new NewExprMatcher(callingAdvisor, methodCall, ast);
      return matcher.matches();
   }

   
}
