/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.test.aop.jdk15.annotated.declare;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;

/**
 * Wraps calls to System.out.println(String), and stores all Strings beginning 
 * with "WARNING:"
 *
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.1.8.2 $
 */
public class SystemOutDecorator extends PrintStream
{
   static PrintStream sysout = null;
   ArrayList warnings = new ArrayList();
   public SystemOutDecorator(OutputStream out)
   {
      super(out);
      sysout = System.out;
   }
   
   public ArrayList getWarnings()
   {
      return warnings;
   }
   
   public void println(String msg)
   {
      super.println(msg);
      if (msg.startsWith("WARNING:"))
      {
         System.out.println(">>>>>");
         super.println(msg);
         System.out.println("<<<<<");
         warnings.add(msg);
      }
   }
   
   public static SystemOutDecorator initialise()
   {
      SystemOutDecorator sys = new SystemOutDecorator(System.out);
      System.setOut(sys);
      return sys;
   }
   
   public void kill()
   {
      System.setOut(sysout);
   }

   String getRidOfAllWhiteSpace(String msg)
   {
      StringBuffer sb = new StringBuffer();
      
      for (int i = 0 ; i < msg.length() ; i++)
      {
         char ch = msg.charAt(i);
         if (ch != '\n' && ch != '\t' && ch != ' ' && ch != '\r')
         {
            sb.append(ch);
         }
      }
      
      return sb.toString();
   }
}
