/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.asynch;


import org.jboss.aop.Dispatcher;
import org.jboss.util.id.GUID;

import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.8.3 $
 */
public class FutureImplJavaUtilConcurrent implements RemotableFuture
{
   private java.util.concurrent.Future result;
   private GUID remoteObjectID;

   public FutureImplJavaUtilConcurrent(java.util.concurrent.Future result)
   {
      this.result = result;
   }

   public void setRemoteObjectID(GUID remoteObjectID)
   {
      this.remoteObjectID = remoteObjectID;
   }

   public void release()
   {
      if (remoteObjectID != null)
      {
         Dispatcher.singleton.unregisterTarget(remoteObjectID);
      }
   }

   public Object get() throws InterruptedException, InvocationTargetException
   {
      try
      {
         Object rtn = result.get();
         release();
         return rtn;
      }
      catch (ExecutionException e)
      {
         release();
         throw new InvocationTargetException(e.getCause());
      }
   }

   public Object get(long milliseconds) throws org.jboss.aspects.asynch.TimeoutException, InterruptedException, InvocationTargetException
   {
      try
      {
         Object rtn = result.get(milliseconds, TimeUnit.MILLISECONDS);
         release();
         return rtn;
      }
      catch (ExecutionException e)
      {
         release();
         throw new InvocationTargetException(e.getCause());
      }
      catch (java.util.concurrent.TimeoutException e)
      {
         throw new org.jboss.aspects.asynch.TimeoutException(e);
      }
   }

   public boolean isDone()
   {
      return result.isDone();
   }

}
