/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.concurrent;

import org.jboss.aop.joinpoint.Invocation;

import java.util.concurrent.locks.ReentrantLock;

/**
 * comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 */
public class MutexAspect
{
   public static class BlockingMutex
   {
      protected MutexLocked props;

      public Object acquire(Invocation invocation) throws Throwable
      {
         MutexedObject obj = (MutexedObject) invocation.getTargetObject();
         ReentrantLock lock = obj.getMutex();
         try
         {
            lock.lockInterruptibly();
         }
         catch (InterruptedException e)
         {
            throw new LockAcquisitionFailureException("Failed to lock for @Mutexed class", e);
         }


         try
         {
            return invocation.invokeNext();
         }
         finally
         {
            lock.unlock();
         }
      }
   }

   public static class TryLockMutex
   {
      protected MutexLocked props;

      public Object acquire(Invocation invocation) throws Throwable
      {
         MutexedObject obj = (MutexedObject) invocation.getTargetObject();
         ReentrantLock lock = obj.getMutex();
         if (!lock.tryLock())
         {
            throw new LockAcquisitionFailureException("Failed to lock for @Mutexed class");
         }

         try
         {
            return invocation.invokeNext();
         }
         finally
         {
            lock.unlock();
         }
      }
   }


   public static class TimeoutMutex
   {
      protected MutexLocked props;

      public TimeoutMutex(MutexLocked props)
      {
         this.props = props;
      }

      public Object acquire(Invocation invocation) throws Throwable
      {
         MutexedObject obj = (MutexedObject) invocation.getTargetObject();
         ReentrantLock lock = obj.getMutex();
         try
         {
            if (!lock.tryLock(props.timeout(), props.unit()))
            {
               throw new LockAcquisitionFailureException("Failed to acquire permit for @Semaphored class.  Timeout reached.");
            }
         }
         catch (InterruptedException e)
         {
            throw new LockAcquisitionFailureException("Failed to acquire permit for @Semaphored class", e);
         }


         try
         {
            return invocation.invokeNext();
         }
         finally
         {
            lock.unlock();
         }
      }
   }


}
