/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.concurrent;

import org.jboss.aop.joinpoint.Invocation;

import java.util.concurrent.locks.ReentrantLock;

/**
 * comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 */
public class MutexedMethodAspect
{
   protected ReentrantLock lock;
   protected MutexedMethod props;

   public MutexedMethodAspect(MutexedMethod m)
   {
      props = m;
      lock = new ReentrantLock(m.isFair());
   }

   public Object acquire(Invocation invocation) throws Throwable
   {
      if (props.timeout() == -1)
      {
            blockIndefinately();
      }
      else if (props.timeout() == 0)
      {
         tryLock();
      }
      else
      {
         tryLockWithTimeout();
      }

      try
      {
         return invocation.invokeNext();
      }
      finally
      {
         lock.unlock();
      }
   }

   protected void blockIndefinately()
   {
      try
      {
         lock.lockInterruptibly();
      }
      catch (InterruptedException e)
      {
         throw new LockAcquisitionFailureException("Failed to lock @MutexedMethod", e);
      }
   }

   protected void tryLock()
   {
      if (!lock.tryLock())
      {
         throw new LockAcquisitionFailureException("Failed to lock @MutexedMethod");
      }
   }

   protected void tryLockWithTimeout()
   {
      try
      {
         if (!lock.tryLock(props.timeout(), props.unit()))
         {
            throw new LockAcquisitionFailureException("Failed to lock @MutexedMethod.  Timeout reached.");
         }
      }
      catch (InterruptedException e)
      {
         throw new LockAcquisitionFailureException("Failed to lock @MutexedMethod", e);
      }
   }
}
