/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.concurrent;

import org.jboss.aop.joinpoint.Invocation;

import java.util.concurrent.Semaphore;

/**
 * comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 */
public class SemaphoreAspect
{
   protected SemaphoreLocked props;

   public SemaphoreAspect(SemaphoreLocked locked)
   {
      this.props = locked;
   }

   public Object acquire(Invocation invocation) throws Throwable
   {
      SemaphoredObject obj = (SemaphoredObject)invocation.getTargetObject();

      if (props.timeout() == -1)
      {
            blockIndefinately(obj.getSemaphore());
      }
      else if (props.timeout() == 0)
      {
         tryLock(obj.getSemaphore());
      }
      else
      {
         tryLockWithTimeout(obj.getSemaphore());
      }

      try
      {
         return invocation.invokeNext();
      }
      finally
      {
         obj.getSemaphore().release(props.permits());
      }
   }

   protected void blockIndefinately(Semaphore semaphore)
   {
      try
      {
         semaphore.acquire(props.permits());
      }
      catch (InterruptedException e)
      {
         throw new LockAcquisitionFailureException("Failed to acquire permit for @Semaphored class", e);
      }
   }

   protected void tryLock(Semaphore semaphore)
   {
      if (!semaphore.tryAcquire(props.permits()))
      {
         throw new LockAcquisitionFailureException("Failed to acquire permit for @Semaphored class");
      }
   }

   protected void tryLockWithTimeout(Semaphore semaphore)
   {
      try
      {
         if (!semaphore.tryAcquire(props.timeout(), props.unit()))
         {
            throw new LockAcquisitionFailureException("Failed to acquire permit for @Semaphored class.  Timeout reached.");
         }
      }
      catch (InterruptedException e)
      {
         throw new LockAcquisitionFailureException("Failed to acquire permit for @Semaphored class", e);
      }
   }
}
