/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.concurrent;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.concurrent.TimeUnit;


/**
 * Only a certain amount of threads can access a @Semaphored object.
 * Acquire a permit when entering a method, release it when leaving.
 *
 * @see Semaphored
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 */
@Target({ElementType.METHOD})
@Retention(RetentionPolicy.RUNTIME)
public @interface SemaphoreLocked
{
   /**
    * Num permits to acquire
    */
   int permits() default 1;
   /**
    * Defaults to -1, block until acquired
    * 0 is just a tryLock
    *
    */
   long timeout() default -1;
   TimeUnit unit() default TimeUnit.MILLISECONDS;
}
