/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.concurrent;

import org.jboss.aop.joinpoint.Invocation;

import java.util.concurrent.Semaphore;

/**
 * comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 */
public class SemaphoredMethodAspect
{
   protected Semaphore semaphore;
   protected SemaphoredMethod props;

   public SemaphoredMethodAspect(SemaphoredMethod m)
   {
      props = m;
      semaphore = new Semaphore(m.permits(), m.isFair());
   }

   public Object acquire(Invocation invocation) throws Throwable
   {
      if (props.timeout() == -1)
      {
            blockIndefinately();
      }
      else if (props.timeout() == 0)
      {
         tryLock();
      }
      else
      {
         tryLockWithTimeout();
      }

      try
      {
         return invocation.invokeNext();
      }
      finally
      {
         semaphore.release();
      }
   }

   protected void blockIndefinately()
   {
      try
      {
         semaphore.acquire();
      }
      catch (InterruptedException e)
      {
         throw new LockAcquisitionFailureException("Failed to acquire permit for @SemaphoredMethod", e);
      }
   }

   protected void tryLock()
   {
      if (!semaphore.tryAcquire())
      {
         throw new LockAcquisitionFailureException("Failed to acquire permit for @SemaphoredMethod");
      }
   }

   protected void tryLockWithTimeout()
   {
      try
      {
         if (!semaphore.tryAcquire(props.permits(), props.timeout(), props.unit()))
         {
            throw new LockAcquisitionFailureException("Failed to acquire permit for @SemaphoredMethod.  Timeout reached.");
         }
      }
      catch (InterruptedException e)
      {
         throw new LockAcquisitionFailureException("Failed to acquire permit for @Semaphored class", e);
      }
   }
}
