/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.asynch;

import EDU.oswego.cs.dl.util.concurrent.FutureResult;
import org.jboss.aop.Dispatcher;
import org.jboss.util.id.GUID;

import java.lang.reflect.InvocationTargetException;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3.8.3 $
 */
public class FutureImpl
implements RemotableFuture
{
   private FutureResult result;
   private GUID remoteObjectID;

   public FutureImpl(FutureResult result)
   {
      this.result = result;
   }

   public void setRemoteObjectID(GUID remoteObjectID)
   {
      this.remoteObjectID = remoteObjectID;
   }

   public void release()
   {
      if (remoteObjectID != null)
      {
         Dispatcher.singleton.unregisterTarget(remoteObjectID);
      }
   }

   public Object get() throws InterruptedException, InvocationTargetException
   {
      try
      {
         Object rtn = result.get();
         release();
         return rtn;
      }
      catch (InvocationTargetException e)
      {
         release();
         throw e;
      }
   }

   public Object get(long milliseconds) throws TimeoutException, InterruptedException, InvocationTargetException
   {
      try
      {
         Object rtn = result.timedGet(milliseconds);
         release();
         return rtn;
      }
      catch (EDU.oswego.cs.dl.util.concurrent.TimeoutException e)
      {
         throw new TimeoutException(e);
      }
      catch (InvocationTargetException e)
      {
         release();
         throw e;
      }
   }

   public boolean isDone()
   {
      return result.isReady();
   }

}
