/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.asynch;

import java.lang.reflect.InvocationTargetException;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.8.3 $
 */
public class FutureProxy implements Future
{
   private Future delegate;
   private boolean released = false;

   public void setDelegate(Future delegate)
   {
      synchronized (this)
      {
         this.delegate = delegate;
         if (released) delegate.release();
         this.notifyAll();
      }
   }

   public void release()
   {
      synchronized (this)
      {
         if (delegate == null)
         {
            released = true;
            return;
         }
      }
      delegate.release();
   }

   public Object get() throws InterruptedException, InvocationTargetException
   {
      synchronized (this)
      {
         if (delegate == null)
            this.wait();
         if (delegate == null) throw new RuntimeException("Failed to get delegate and timed out");
         return delegate.get();
      }
   }

   public Object get(long milliseconds) throws TimeoutException, InterruptedException, InvocationTargetException
   {
      synchronized (this)
      {
         if (delegate == null)
         {
            this.wait(milliseconds);
            if (delegate == null) throw new TimeoutException("Failed to get delegate and timed out");
         }
      }
      return delegate.get(milliseconds);
   }

   public boolean isDone()
   {
      return delegate == null || delegate.isDone();
   }

}
