/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.dbc.condition;

import java.lang.reflect.Constructor;

import org.jboss.aop.joinpoint.Invocation;

/**
 *
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.1.8.4 $
 */
public class ConstructorCondition extends ExecutableCondition
{
   Constructor constructor;
   
   public ConstructorCondition(Constructor constructor, String condExpr)
   {
      super(condExpr, constructor.getClass(), false);
      this.constructor = constructor;
   }
   
   public boolean equals(Object o)
   {
      if (o instanceof ConstructorCondition)
      {
         ConstructorCondition cc = (ConstructorCondition)o;
         if (cc.constructor.equals(constructor))
         {
            return super.equals(o);
         }
      }
      return false;
   }

   // ExecutableCondition implementation -------------------------------------
   protected Object executableObject()
   {
      return constructor;
   }

   protected Class[] parameterTypes()
   {
      return constructor.getParameterTypes();
   }
   
   protected boolean isPredefinedToken(String token)
   {
      return token.equals(Condition.TARGET);
   }
   
   protected Object getValueForPredefinedToken(Invocation invocation, Object rtn, String token)
   {
      if (token.equals(Condition.TARGET))
      {
         //For constructors the $tgt token corresponds to the return of the method
         return rtn;
      }

      return null;
   }
}
