/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.tx;

import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.tm.TransactionPropagationContextUtil;
import org.jboss.tm.TxManager;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3.2.4 $
 */
public class TxPropagationInterceptor implements Interceptor
{
   private TransactionManager tm;

   public TxPropagationInterceptor(TransactionManager tm)
   {
      this.tm = tm;
   }

   public TxPropagationInterceptor()
   {
      tm = TxManager.getInstance();
   }

   public String getName()
   {
      return "TxPropagationInterceptor";
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      Object tpc = invocation.getMetaData(ClientTxPropagationInterceptor.TRANSACTION_PROPAGATION_CONTEXT, ClientTxPropagationInterceptor.TRANSACTION_PROPAGATION_CONTEXT);
      if (tpc != null)
      {
         Transaction tx = tm.getTransaction();
         if (tx != null) throw new RuntimeException("cannot import a transaction context when a transaction is already associated with the thread");
         Transaction importedTx = TransactionPropagationContextUtil.getTPCImporter().importTransactionPropagationContext(tpc);
         tm.resume(importedTx);
         try
         {
            return invocation.invokeNext();
         }
         finally
         {
            tm.suspend();
         }
      }
      else
      {
         return invocation.invokeNext();
      }
   }
}
