/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package test.dbc.office;

import java.util.ArrayList;

/**
 *
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.1.8.3 $
 * @@org.jboss.aspects.dbc.Dbc
 * @@org.jboss.aspects.dbc.Invariant ({"$tgt.computers != null", "$tgt.developers != null", "forall test.dbc.office.Computer c in $tgt.computers | c != null", "forall d in $tgt.developers | d != null"})
 */
public class OfficeManager
{
   ArrayList computers = new ArrayList();
   ArrayList developers = new ArrayList();
   
   /**
    * PostCond: The computer should be unassigned after adding
    * @@org.jboss.aspects.dbc.PostCond ({"exists test.dbc.office.Computer c in $tgt.computers | c.getDeveloper() == null && c == $rtn"})
    */
   public Computer createComputer(String name)
   {
      Computer computer = new Computer(name);
      computers.add(computer);
      return computer;
   }
   
   /**
    * PostCond: The developer should not have a computer after adding
    * @@org.jboss.aspects.dbc.PostCond ({"exists test.dbc.office.Developer d in $tgt.developers | d.getComputer() == null && d == $rtn"})
    */
   public Developer createDeveloper(String name)
   {
      Developer developer = new Developer(name);
      developers.add(developer);
      return developer;
   }
   
   /**
    * PreCond: The computer and developer must both be not-null, and not previously assigned
    * PostCond: Make sure that all developers have a computer, and that that computer is
    * associated with the developer in question
    * 
    * @@org.jboss.aspects.dbc.PreCond ({"$0 != null", "$1 != null", "exists test.dbc.office.Computer c in $tgt.computers | c.getDeveloper() == null && c == $0", "exists test.dbc.office.Developer d in $tgt.developers | d.getComputer() == null && d == $1"})
    * @@org.jboss.aspects.dbc.PostCond ({"forall d in $tgt.developers | exists c in $tgt.computers | (c == d.getComputer() && d == c.getDeveloper())"})
    */
   public void assignComputer(Computer computer, Developer developer)
   {
      System.out.println("========================= SET DEVELOPER ==============================");
      computer.setDeveloper(developer);
      System.out.println("========================= SET COMPUTER ==============================");
      developer.setComputer(computer);
   }
   
}
