/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.console.plugins.monitor;

import java.io.IOException;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.monitor.services.ActiveAlarmTableMBean;
import org.jboss.mx.util.MBeanServerLocator;

/**
 * @author  <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>.
 * @version $Revision: 1.1.2.1 $
 */
public class AcknowledgeActiveAlarmsServlet extends HttpServlet
{
   protected void doGet(HttpServletRequest req, HttpServletResponse resp)
           throws ServletException, IOException
   {
      doit(req, resp);
   }

   protected void doPost(HttpServletRequest req, HttpServletResponse resp)
           throws ServletException, IOException
   {
      doit(req, resp);
   }

   protected void doit(HttpServletRequest req, HttpServletResponse resp)
           throws ServletException, IOException
   {
      try
      {
         MBeanServer server = MBeanServerLocator.locateJBoss();
         ObjectName target = ActiveAlarmTableMBean.OBJECT_NAME;

         // use the alarm table serverId for 'system' 
         String system = (String)server.getAttribute(target, "ServerId");
         // use the user.name environment property for user
         String user = System.getProperty("user.name");
         
         String alarmId = req.getParameter("alarmId").trim();
         
         if (alarmId.equals("*"))
         {
            server.invoke(
                  target,
                  "acknowledgeAll",
                  new Object[] { user, system },
                  new String[] { "java.lang.String", "java.lang.String" });
         }
         else
         {
            server.invoke(
                  target,
                  "acknowledge",
                  new Object[] { alarmId, user, system },
                  new String[] { "java.lang.String", "java.lang.String", "java.lang.String" });
         }
      }
      catch (Exception ex)
      {
         req.setAttribute("error", "Error acknowledging alarms: " + ex.toString());
      }
      req.getRequestDispatcher("/listActiveAlarmTable.jsp").forward(req, resp);
   }

}
