/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.deployment.spi.configurations;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.jar.JarOutputStream;

import javax.enterprise.deploy.model.DDBeanRoot;
import javax.enterprise.deploy.model.DeployableObject;
import javax.enterprise.deploy.spi.DConfigBeanRoot;
import javax.enterprise.deploy.spi.DeploymentConfiguration;
import javax.enterprise.deploy.spi.exceptions.BeanNotFoundException;
import javax.enterprise.deploy.spi.exceptions.ConfigurationException;

import org.jboss.deployment.spi.DeploymentMetaData;
import org.jboss.deployment.spi.JarUtils;
import org.jboss.deployment.spi.beans.JBossConfigBeanProxy;
import org.jboss.deployment.spi.beans.WarConfigBeanRoot;

/**
 * The war configuration container.
 * 
 * @author Rob Stryker
 * @version $Revision: 1.4 $
 */
public class WarConfiguration implements DeploymentConfiguration
{

   private DeployableObject deployable;
   private HashMap configBeans;


   public WarConfiguration(DeployableObject deployable)
   {
      this.deployable = deployable;
      configBeans = new HashMap(); // maps filename to dconfigbean

   }


   public DeployableObject getDeployableObject()
   {
      return this.deployable;
   }

   public DConfigBeanRoot getDConfigBeanRoot(DDBeanRoot dd) throws ConfigurationException
   {
      // If they give us web.xml, return our jboss-web.xml config bean
      if (configBeans.containsKey(dd.getFilename()))
      {
         return (DConfigBeanRoot) configBeans.get(dd.getFilename());
      }
		
      // Not found, so create it. (lazy initializing)
      if (dd.getFilename().equals("WEB-INF/web.xml"))
      {
         DConfigBeanRoot retval = new WarConfigBeanRoot(dd, deployable);
         configBeans.put(dd.getFilename(), retval);
         return retval;
      }
		
		
      // if they give us some other standard bean, return the jboss specific
      // None implemented as of now
      return null;
   }

   public void removeDConfigBean(DConfigBeanRoot bean) throws BeanNotFoundException
   {
      String key = bean.getDDBean().getRoot().getFilename();
      if (configBeans.containsKey(key))
      {
         System.out.println("its here... not anymore");
         configBeans.remove(key);
      }
      else
      {
         throw new BeanNotFoundException("BNF");
      }
   }


   public void save(OutputStream stream) throws ConfigurationException
   {
      JarOutputStream jos = null;
		
      // Setup deployment plan meta data with propriatary descriptor (jboss-web.xml)
      DeploymentMetaData metaData = new DeploymentMetaData("WRONG.war");


      try
      {
         jos = new JarOutputStream(stream);
      }
      catch (Exception e)
      {
         return;
      }
      if (jos == null) return;

      Iterator setIterator = configBeans.keySet().iterator();
      while (setIterator.hasNext())
      {
         String key = (String) setIterator.next();
         JBossConfigBeanProxy val = (JBossConfigBeanProxy) configBeans.get(key);
         val.save(jos, metaData);
      }
      try
      {
         String metaStr = metaData.toXMLString();
         JarUtils.addJarEntry(jos, DeploymentMetaData.ENTRY_NAME, new ByteArrayInputStream(metaStr.getBytes()));
         jos.flush();
         jos.close();
      }
      catch (Exception e)
      {
         System.out.println("config IO exception error: " + e.getMessage());
      }
   }

   public DConfigBeanRoot restoreDConfigBean(InputStream arg0, DDBeanRoot arg1) throws ConfigurationException
   {
      return null;
   }

   public void saveDConfigBean(OutputStream arg0, DConfigBeanRoot arg1) throws ConfigurationException
   {
   }

   public void restore(InputStream arg0) throws ConfigurationException
   {
   }

}
