/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3;

import org.jboss.ejb3.injection.Injector;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3.2.4 $
 */
public abstract class AbstractPool implements Pool
{
   protected Class beanClass;
   protected Class contextClass;
   protected Injector[] injectors;
   protected Container container;

   public AbstractPool()
   {

   }

   public void initialize(Container container, Class contextClass, Class beanClass, int maxSize, long timeout)
   {
      this.beanClass = beanClass;
      this.contextClass = contextClass;
      this.container = container;
   }

   protected BeanContext create()
   {
      Object bean;
      BeanContext ctx;
      try
      {
         bean = container.construct();
         ctx = (BeanContext) contextClass.newInstance();
         ctx.setContainer(container);
         ctx.setInstance(bean);
      }
      catch (InstantiationException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
      catch (IllegalAccessException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
      if (injectors != null)
      {
         for (int i = 0; i < injectors.length; i++)
         {
            injectors[i].inject(ctx);
         }
      }

      //TODO This needs to be reimplemented as replacement for create() on home interface
      container.invokeInit(bean);

      container.invokePostConstruct(bean);
      return ctx;
   }

   protected BeanContext create(Class[] initTypes, Object[] initValues)
   {
      Object bean;
      BeanContext ctx;
      try
      {
         bean = container.construct();
         ctx = (BeanContext) contextClass.newInstance();
         ctx.setContainer(container);
         ctx.setInstance(bean);
      }
      catch (InstantiationException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
      catch (IllegalAccessException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
      if (injectors != null)
      {
         for (int i = 0; i < injectors.length; i++)
         {
            injectors[i].inject(ctx);
         }
      }

      //TODO This needs to be reimplemented as replacement for create() on home interface
      container.invokeInit(bean, initTypes, initValues);

      container.invokePostConstruct(bean);
      return ctx;
   }

   public void remove(BeanContext ctx)
   {
      Object bean = ctx.getInstance();
      container.invokePreDestroy(bean);
      ctx.remove();
   }

   public void setInjectors(Injector[] injectors)
   {
      this.injectors = injectors;
   }
}
