/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3;

import java.util.Hashtable;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javassist.bytecode.AnnotationsAttribute;
import javassist.bytecode.annotation.Annotation;
import javassist.bytecode.annotation.StringMemberValue;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.7.2.4 $
 */
public class EJB3Util
{
   public static boolean isStateless(AnnotationsAttribute group)
   {
      return group.getAnnotation(javax.ejb.Stateless.class.getName()) != null;
   }

   public static boolean isStatefulSession(AnnotationsAttribute group)
   {
      return group.getAnnotation(javax.ejb.Stateful.class.getName()) != null;
   }

   public static boolean isEntity(AnnotationsAttribute group)
   {
      return group.getAnnotation(javax.persistence.Entity.class.getName()) != null;
   }

   public static boolean isMessageDriven(AnnotationsAttribute group)
   {
      return group.getAnnotation(javax.ejb.MessageDriven.class.getName()) != null;
   }

   public static boolean isService(AnnotationsAttribute group)
   {
      return group.getAnnotation(org.jboss.annotation.ejb.Service.class.getName()) != null;
   }

   public static boolean isConsumer(AnnotationsAttribute group)
   {
      return group.getAnnotation(org.jboss.annotation.ejb.Consumer.class.getName()) != null;
   }

   public static String getAspectDomain(AnnotationsAttribute visible, String defaultContainerName)
   {
      if (visible != null )
      {
         Annotation dinfo = visible.getAnnotation(org.jboss.annotation.ejb.AspectDomain.class
                                      .getName());
         if (dinfo != null)
         {
            StringMemberValue dmv = (StringMemberValue) dinfo
                    .getMemberValue("value");
            if (dmv != null)
            {
               return dmv.getValue();
            }
         }
      }
      
      return defaultContainerName;
   }

   public static InitialContext getInitialContext(Hashtable props)
           throws NamingException
   {
      InitialContext ctx = null;
      if (props != null)
      {
         ctx = new InitialContext(props);
      }
      else ctx = new InitialContext();
      return ctx;
   }
}


