/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3;

import java.util.Hashtable;
import java.util.LinkedList;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.spi.ObjectFactory;
import org.jnp.interfaces.NamingContext;
import org.jnp.server.NamingServer;

/**
 * Implementation of "java:comp" namespace factory. The context is associated
 * with the thread class loader.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.2.4 $
 */
public class ThreadLocalENCFactory
        implements ObjectFactory
{
   private static ThreadLocal enc = new ThreadLocal();
   private static ThreadLocal stack = new ThreadLocal();

   public static Context create(Context parent) throws Exception
   {
      NamingServer srv = new NamingServer();
      return new NamingContext(parent.getEnvironment(), null, srv);
   }

   public static void push(Context ctx)
   {
      if (enc.get() == null)
      {
         enc.set(ctx);
         return;
      }
      LinkedList currentStack = (LinkedList) stack.get();
      if (currentStack == null)
      {
         currentStack = new LinkedList();
         stack.set(currentStack);
      }
      currentStack.addLast(enc.get());
      enc.set(ctx);
   }

   public static void pop()
   {
      LinkedList currentStack = (LinkedList) stack.get();
      if (currentStack == null)
      {
         enc.set(null);
         return;
      }
      if (currentStack.size() == 0)
      {
         enc.set(null);
         return;
      }
      Object previous = currentStack.removeLast();
      enc.set(previous);
   }
   // Constructors --------------------------------------------------

   // Public --------------------------------------------------------

   // ObjectFactory implementation ----------------------------------
   public Object getObjectInstance(Object obj, Name name, Context nameCtx,
                                   Hashtable environment)
           throws Exception
   {
      return enc.get();
   }

}
