/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.cache;

import java.util.HashMap;
import javax.ejb.EJBException;
import javax.ejb.EJBNoSuchObjectException;
import org.jboss.ejb3.Container;
import org.jboss.ejb3.Pool;
import org.jboss.ejb3.stateful.StatefulBeanContext;
import org.jboss.util.id.GUID;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.3 $
 */
public class NoPassivationCache implements StatefulCache
{
   private Pool pool;
   private HashMap cacheMap;

   public void initialize(Container container) throws Exception
   {
      this.pool = container.getPool();
      cacheMap = new HashMap();
   }

   public NoPassivationCache()
   {
   }

   public void start()
   {
   }

   public void stop()
   {
      synchronized (cacheMap)
      {
         cacheMap.clear();
      }
   }

   public StatefulBeanContext create()
   {
      StatefulBeanContext ctx = null;
      try
      {
         GUID guid = new GUID();
         ctx = (StatefulBeanContext) pool.get();
         ctx.setId(guid);
         synchronized (cacheMap)
         {
            cacheMap.put(guid, ctx);
         }
      }
      catch (Exception e)
      {
         e.printStackTrace();
         throw new EJBException(e);
      }
      return ctx;
   }

   public StatefulBeanContext create(Class[] initTypes, Object[] initValues)
   {
      StatefulBeanContext ctx = null;
      try
      {
         GUID guid = new GUID();
         ctx = (StatefulBeanContext) pool.get(initTypes, initValues);
         ctx.setId(guid);
         synchronized (cacheMap)
         {
            cacheMap.put(guid, ctx);
         }
      }
      catch (Exception e)
      {
         throw new EJBException(e);
      }
      return ctx;
   }

   public StatefulBeanContext get(Object key) throws EJBException
   {
      StatefulBeanContext entry = null;
      synchronized (cacheMap)
      {
         entry = (StatefulBeanContext) cacheMap.get(key);
      }
      if (entry == null)
      {
         throw new EJBNoSuchObjectException("Could not find Stateful bean: " + key);
      }
      entry.inUse = true;
      entry.lastUsed = System.currentTimeMillis();
      return entry;
   }

   public void finished(StatefulBeanContext ctx)
   {
      synchronized (ctx)
      {
         ctx.inUse = false;
         ctx.lastUsed = System.currentTimeMillis();
      }
   }

   public void remove(Object key)
   {
      StatefulBeanContext ctx = null;
      synchronized (cacheMap)
      {
         ctx = (StatefulBeanContext) cacheMap.remove(key);
      }
      if (ctx != null) pool.remove(ctx);
   }


}
