/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.cache;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.ejb3.stateful.StatefulBeanContext;
import org.jboss.ejb3.stateful.StatefulContainer;
import org.jboss.ejb3.stateful.StatefulContainerInvocation;

/**
 * Replicate SFSB if it is modified
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.4.3 $
 */
public class StatefulReplicationInterceptor implements Interceptor
{
   public String getName()
   {
      return this.getClass().getName();
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      Object rtn = invocation.invokeNext();

      Object obj = invocation.getTargetObject();

      if (obj instanceof Optimized)
      {
         if (!((Optimized) obj).isModified())
         {
            return rtn;
         }
      }
      StatefulContainerInvocation ejb = (StatefulContainerInvocation) invocation;
      StatefulBeanContext ctx = (StatefulBeanContext) ejb.getBeanContext();
      StatefulContainer container = (StatefulContainer) ejb.getAdvisor();
      ClusteredStatefulCache cache = (ClusteredStatefulCache) container.getCache();
      cache.replicate(ctx);
      return rtn;
   }
}
