/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.cache.simple;

import org.jboss.ejb3.Container;
import org.jboss.ejb3.stateful.StatefulBeanContext;
import org.jboss.logging.Logger;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.6.3 $
 */
public interface StatefulSessionPersistenceManager
{
   Logger log = Logger.getLogger(StatefulSessionFilePersistenceManager.class);
   /**
    * The default store directory name ("<tt>sessions</tt>").
    */
   String DEFAULT_STORE_DIRECTORY_NAME = "sessions";

   /**
    * Restores session state from the serialized file & invokes
    * {@link javax.ejb.SessionBean#ejbActivate} on the target bean.
    */
   StatefulBeanContext activateSession(Object id);

   /**
    * Invokes {@link javax.ejb.SessionBean#ejbPassivate} on the target bean and saves the
    * state of the session to a file.
    */
   void passivateSession(StatefulBeanContext ctx);

   /**
    * Removes the saved state file (if any) for the given session id.
    */
   void removePassivated(Object id);

   void destroy() throws Exception;

   public void initialize(Container container) throws Exception;
}
