/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.cache.tree;

import java.util.Properties;
import org.jboss.cache.Fqn;
import org.jboss.cache.Node;
import org.jboss.cache.TreeCache;
import org.jboss.cache.eviction.LRUPolicy;
import org.jboss.cache.eviction.Region;
import org.jboss.cache.eviction.RegionNameConflictException;
import org.jboss.cache.loader.FileCacheLoader;
import org.jboss.system.server.ServerConfig;
import org.jboss.util.xml.XmlHelper;
import org.w3c.dom.Element;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.6.3 $
 */
public class PassivationEvictionPolicy extends LRUPolicy
{
   protected FileCacheLoader fileCacheLoader = new FileCacheLoader();

   public void createRegion(String name, int maxSize, long timeToLive)
   {
      if (regionManager_.hasRegion(name)) return;
      try
      {
         Region region = regionManager_.createRegion(name, getEvictionAlgorithm());
         region.setTimeToLiveSeconds(timeToLive);
         region.setMaxNodes(maxSize);
      }
      catch (RegionNameConflictException e)
      {
         throw new RuntimeException(e);
      }
   }

   public void removeRegion(String name)
   {
      regionManager_.removeRegion(name);
   }

   public void superEvict(Fqn fqn) throws Exception
   {
      super.evict(fqn);
   }

   public void evict(Fqn fqn) throws Exception
   {
      Node node = cache_.get(fqn);
      //System.out.println("EVICTING: " + fqn.toString() + " size: " + fqn.size() + " 0: " + fqn.get(0));
      fileCacheLoader.put(fqn, node.getData(), true);
      super.evict(fqn);
   }

   public void configure(TreeCache cache)
   {
      super.configure(cache);
      Element elem = cache.getEvictionPolicyConfig();
      try
      {
         String name = XmlHelper.getUniqueChildContent(elem, "name");
         String filename = System.getProperty(ServerConfig.SERVER_TEMP_DIR) + "/" + name;
         Properties p = new Properties();
         p.put("location", filename);
         fileCacheLoader.setConfig(p);
         fileCacheLoader.create();
         fileCacheLoader.start();
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);
      }
   }
}
