/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.callback;

import java.util.HashMap;
import javax.ejb.CallbackListener;
import javax.ejb.PostActivate;
import javax.ejb.PostConstruct;
import javax.ejb.PreDestroy;
import javax.ejb.PrePassivate;
import javax.ejb.Timeout;
import org.jboss.aop.Advisor;
import org.jboss.logging.Logger;

/**
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.6.2.3 $
 */
public class SessionCallbackHandler
{
   private static final Logger log = Logger.getLogger(SessionCallbackHandler.class);

   private Advisor beanAdvisor;
   private Class beanClass;
   private Class listener;

   HashMap<Class, Callback> callbacks = new HashMap<Class, Callback>();

   public SessionCallbackHandler(Advisor beanAdvisor, Class[] validCallbacks) throws Exception
   {
      this.beanAdvisor = beanAdvisor;
      beanClass = beanAdvisor.getClazz();
      
      CallbackListener cl = (CallbackListener) beanAdvisor.resolveAnnotation(CallbackListener.class);
      if (cl != null)
      {
         listener = cl.value();
      }

      for (int i = 0; i < validCallbacks.length; i++)
      {
         Callback callback = resolveCallback(beanAdvisor, validCallbacks[i]);
         if (callback != null)
         {
            callbacks.put(validCallbacks[i], callback);
         }
         else
         {
            Object o = beanAdvisor.resolveAnnotation(validCallbacks[i]);
         }
      }
   }

   public void postConstruct(Object bean)
   {
      callback(PostConstruct.class, bean);
   }

   public void preDestroy(Object bean)
   {
      callback(PreDestroy.class, bean);
   }

   public void postActivate(Object bean)
   {
      callback(PostActivate.class, bean);
   }

   public void prePassivate(Object bean)
   {
      callback(PrePassivate.class, bean);
   }

   public Callback getTimeoutCallback()
   {
      return getCallback(Timeout.class);
   }

   public Callback getCallback(Class callback)
   {
      return callbacks.get(callback);
   }

   private void callback(Class callback, Object bean)
   {
      try
      {
         Callback cb = callbacks.get(callback);
         if (cb != null)
         {
            cb.invoke(bean);
         }
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);
      }
   }

   private Callback resolveCallback(Advisor beanAdvisor, Class annotation) throws Exception
   {
      Class beanClass = beanAdvisor.getClazz();
      return CallbackResolver.resolveCallback(beanAdvisor, beanClass, annotation, listener);
   }
}
