/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.entity;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.transaction.TransactionManager;
import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.ejb3.stateful.StatefulBeanContext;
import org.jboss.ejb3.stateful.StatefulContainerInvocation;
import org.jboss.ejb3.tx.TxUtil;
import org.jboss.logging.Logger;

/**
 *
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.4 $
 */
public class ExtendedPersistenceContextPropagationInterceptor implements Interceptor
{
   private static final Logger log = Logger.getLogger(ExtendedPersistenceContextPropagationInterceptor.class);

   public String getName()
   {
      return this.getClass().getName();
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      log.debug("++++ LongLivedSessionPropagationInterceptor");
      StatefulContainerInvocation ejb = (StatefulContainerInvocation) invocation;
      StatefulBeanContext ctx = (StatefulBeanContext) ejb.getBeanContext();

      Map extendedPCs = ctx.getMetaData().tag(ManagedEntityManagerFactory.LONG_LIVED_SESSION);
      if (extendedPCs == null)
      {
         return invocation.invokeNext();
      }

      Map<String, ExtendedPersistenceContext> pcMap = new HashMap<String, ExtendedPersistenceContext>();
      Iterator it = extendedPCs.keySet().iterator();
      while (it.hasNext())
      {
         String name = (String) it.next();
         ExtendedPersistenceContext pc = (ExtendedPersistenceContext) ctx.getMetaData().getMetaData(ManagedEntityManagerFactory.LONG_LIVED_SESSION, name);
         pcMap.put(name, pc);
      }

      Object old = ManagedEntityManagerFactory.longLivedSession.get();
      try
      {
         ManagedEntityManagerFactory.longLivedSession.set(pcMap);
         TransactionManager tm = TxUtil.getTransactionManager();
         if (tm.getTransaction() != null)
         {
            for (ExtendedPersistenceContext pc : pcMap.values())
            {
               // make sure transaction is associated.
               pc.getFactory().registerPersistenceContext();
            }
         }

         return invocation.invokeNext();
      }
      finally
      {
         ManagedEntityManagerFactory.longLivedSession.set(old);
      }
   }
}
