/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.entity;

import javax.persistence.FlushModeType;
import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.tm.TransactionLocal;
import org.jboss.tm.TxManager;

/**
 * comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 */
public class FlushModeInterceptor implements Interceptor
{
   private FlushModeType mode;

   public FlushModeInterceptor(FlushModeType mode)
   {
      this.mode = mode;
   }

   public String getName()
   {
      return FlushModeInterceptor.class.getName();
   }


   protected static ThreadLocal flushMode = new ThreadLocal();
   // todo this is a bad way to initialize and will not allow us to plug in another transaction manager
   protected static TransactionLocal txFlushMode = new TransactionLocal(TxManager.getInstance());

   public static FlushModeType getFlushMode()
   {
      FlushModeType mode = (FlushModeType) flushMode.get();
      if (mode == null) mode = FlushModeType.AUTO;
      return mode;
   }

   public static FlushModeType getTxFlushMode()
   {
      FlushModeType mode = (FlushModeType) txFlushMode.get();
      if (mode == null) mode = FlushModeType.AUTO;
      return mode;
   }


   public Object invoke(Invocation invocation) throws Throwable
   {
      flushMode.set(mode);
      if (txFlushMode.getTransaction() != null) txFlushMode.set(mode);
      return invocation.invokeNext();
   }
}
