/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.entity;

import java.io.Serializable;
import java.util.Hashtable;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.Query;
import org.hibernate.Session;
import org.hibernate.ejb.HibernateEntityManager;
import org.jboss.ejb3.EJB3Util;


/**
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.5 $
 */
public class InjectedEntityManager implements EntityManager, HibernateSession, Serializable
{
   private transient ManagedEntityManagerFactory factory;
   private String jndiName;
   private Hashtable initialContextProperties;

   public Session getHibernateSession()
   {
      if (getSession() instanceof HibernateEntityManager)
      {
         return ((HibernateEntityManager) getSession()).getSession();
      }
      throw new RuntimeException("ILLEGAL ACTION:  Not a Hibernate pe" +
                                 "rsistence provider");
   }

   public InjectedEntityManager(ManagedEntityManagerFactory factory, Hashtable initialContextProperties)
   {
      if (factory == null) throw new NullPointerException("factory must not be null");
      this.factory = factory;
      this.jndiName = factory.getJndiName();
      this.initialContextProperties = initialContextProperties;
   }

   public InjectedEntityManager()
   {

   }

   public <T> T getReference(Class<T> entityClass, Object primaryKey)
   {
      return getSession().getReference(entityClass, primaryKey);
   }

   public void setFlushMode(FlushModeType flushMode)
   {
      getSession().setFlushMode(flushMode);
   }

   public Query createQuery(String ejbqlString)
   {
      return getSession().createQuery(ejbqlString);
   }

   public Query createNamedQuery(String name)
   {
      return getSession().createNamedQuery(name);
   }

   public Query createNativeQuery(String sqlString)
   {
      return getSession().createNativeQuery(sqlString);
   }

   public Query createNativeQuery(String sqlString, Class resultClass)
   {
      return getSession().createNativeQuery(sqlString, resultClass);
   }

   public Query createNativeQuery(String sqlString, String resultSetMapping)
   {
      return getSession().createNativeQuery(sqlString, resultSetMapping);
   }

   public <A> A find(Class<A> entityClass, Object primaryKey)
   {
      return getSession().find(entityClass, primaryKey);
   }

   public void persist(Object entity)
   {
      getSession().persist(entity);
   }

   public <A> A merge(A entity)
   {
      return (A) getSession().merge(entity);
   }

   public void remove(Object entity)
   {
      getSession().remove(entity);
   }

   public void refresh(Object entity)
   {
      getSession().refresh(entity);
   }

   public boolean contains(Object entity)
   {
      return getSession().contains(entity);
   }

   public void flush()
   {
      getSession().flush();
   }

   public void close()
   {
      throw new IllegalStateException("Illegal to call this method from injected, managed EntityManager");
   }

   public boolean isOpen()
   {
      throw new IllegalStateException("Illegal to call this method from injected, managed EntityManager");
   }

   public EntityTransaction getTransaction()
   {
      throw new IllegalStateException("Illegal to call this method from injected, managed EntityManager");
   }

   protected EntityManager getSession()
   {
      if (factory == null)
      {
         try
         {
            InitialContext ctx = EJB3Util.getInitialContext(initialContextProperties);
            factory = (ManagedEntityManagerFactory) ctx.lookup(jndiName);
         }
         catch (NamingException e)
         {
            throw new RuntimeException(e);
         }
      }
      return factory.getSession();
   }

}
