/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.entity;

import java.io.Serializable;
import java.sql.Connection;
import java.util.Hashtable;
import java.util.Map;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.persistence.EntityManagerFactory;
import org.hibernate.HibernateException;
import org.hibernate.Interceptor;
import org.hibernate.SessionFactory;
import org.hibernate.StatelessSession;
import org.hibernate.classic.Session;
import org.hibernate.ejb.HibernateEntityManagerFactory;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.metadata.CollectionMetadata;
import org.hibernate.stat.Statistics;
import org.jboss.ejb3.EJB3Util;
import org.jboss.ejb3.EJBContainer;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.3 $
 */
public class InjectedSessionFactory implements SessionFactory, Serializable
{
   private transient EntityManagerFactory delegate;
   private String jndiName;
   private Hashtable initialContextProperties;

   public InjectedSessionFactory(EntityManagerFactory delegate, String jndiName, Hashtable initialContextProperties)
   {
      this.delegate = delegate;
      this.jndiName = jndiName;
      this.initialContextProperties = initialContextProperties;
   }


   public InjectedSessionFactory(EntityManagerFactory delegate, Hashtable initialContextProperties)
   {
      this(delegate, EJBContainer.ENC_CTX_NAME + "/" + EJBContainer.ENTITY_MANAGER_FACTORY, initialContextProperties);
   }

   private EntityManagerFactory getDelegate()
   {
      if (delegate == null)
      {
         try
         {
            InitialContext ctx = EJB3Util.getInitialContext(initialContextProperties);
            delegate = (EntityManagerFactory) ctx.lookup(jndiName);
         }
         catch (NamingException e)
         {
            throw new RuntimeException(e);
         }
      }
      return delegate;
   }

   private SessionFactory getSessionFactory()
   {
      return ((HibernateEntityManagerFactory)getDelegate()).getSessionFactory();
   }

   public Session openSession(Connection connection)
   {
      return getSessionFactory().openSession(connection);
   }

   public Session openSession(Interceptor interceptor)
   throws HibernateException
   {
      return getSessionFactory().openSession(interceptor);
   }

   public Session openSession(Connection connection, Interceptor interceptor)
   {
      return getSessionFactory().openSession(connection, interceptor);
   }

   public Session openSession()
   throws HibernateException
   {
      return getSessionFactory().openSession();
   }

   public Session getCurrentSession()
   throws HibernateException
   {
      return getSessionFactory().getCurrentSession();
   }

   public ClassMetadata getClassMetadata(Class persistentClass)
   throws HibernateException
   {
      return getSessionFactory().getClassMetadata(persistentClass);
   }

   public ClassMetadata getClassMetadata(String entityName)
   throws HibernateException
   {
      return getSessionFactory().getClassMetadata(entityName);
   }

   public CollectionMetadata getCollectionMetadata(String roleName)
   throws HibernateException
   {
      return getSessionFactory().getCollectionMetadata(roleName);
   }

   public Map getAllClassMetadata()
   throws HibernateException
   {
      return getSessionFactory().getAllClassMetadata();
   }

   public Map getAllCollectionMetadata()
   throws HibernateException
   {
      return getSessionFactory().getAllCollectionMetadata();
   }

   public Statistics getStatistics()
   {
      return getSessionFactory().getStatistics();
   }

   public void close()
   throws HibernateException
   {
      throw new IllegalStateException("It is illegal to close an injected SessionFactory");
   }

   public boolean isClosed()
   {
      return getSessionFactory().isClosed();
   }

   public void evict(Class persistentClass)
   throws HibernateException
   {
      getSessionFactory().evict(persistentClass);
   }

   public void evict(Class persistentClass, Serializable id)
   throws HibernateException
   {
      getSessionFactory().evict(persistentClass, id);
   }

   public void evictEntity(String entityName)
   throws HibernateException
   {
      getSessionFactory().evictEntity(entityName);
   }

   public void evictEntity(String entityName, Serializable id)
   throws HibernateException
   {
      getSessionFactory().evictEntity(entityName, id);
   }

   public void evictCollection(String roleName)
   throws HibernateException
   {
      getSessionFactory().evictCollection(roleName);
   }

   public void evictCollection(String roleName, Serializable id)
   throws HibernateException
   {
      getSessionFactory().evictCollection(roleName, id);
   }

   public void evictQueries()
   throws HibernateException
   {
      getSessionFactory().evictQueries();
   }

   public void evictQueries(String cacheRegion)
   throws HibernateException
   {
      getSessionFactory().evictQueries(cacheRegion);
   }

   public StatelessSession openStatelessSession()
   {
      return getSessionFactory().openStatelessSession();
   }

   public StatelessSession openStatelessSession(Connection connection)
   {
      return getSessionFactory().openStatelessSession(connection);
   }

   public Reference getReference()
   throws NamingException
   {
      return getSessionFactory().getReference();
   }

}
