/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.injection;

import java.lang.reflect.Field;
import javax.naming.Context;
import javax.naming.NamingException;
import org.jboss.ejb3.BeanContext;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3.2.4 $
 */
public class JndiFieldInjector implements Injector
{
   private Field field;
   private String jndiName;
   private Context ctx;

   public JndiFieldInjector(Field field, String jndiName, Context ctx)
   {
      this.field = field;
      this.field.setAccessible(true);
      this.jndiName = jndiName;
      this.ctx = ctx;
   }

   public JndiFieldInjector(Field field, Context ctx)
   {
      this(field, field.getName(), ctx);
   }

   public void inject(BeanContext bctx)
   {
      Object dependency = null;
      try
      {
         dependency = ctx.lookup(jndiName);
      }
      catch (NamingException e)
      {
         throw new RuntimeException("Unable to inject jndi dependency: " + jndiName + " into field " + field, e);
      }
      try
      {
         field.set(bctx.getInstance(), dependency);
      }
      catch (IllegalArgumentException e)
      {
         String type = "UNKNOWN";
         if (dependency != null) type = dependency.getClass().getName();
         throw new RuntimeException("Non matching type for inject of field: " + field + " for type: " + type, e);
      }
      catch (IllegalAccessException e)
      {
         throw new RuntimeException(e);
      }
   }
}
