/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.injection;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import javax.naming.InitialContext;
import org.jboss.annotation.JndiInject;
import org.jboss.aop.Advisor;
import org.jboss.ejb3.Container;

/**
 * Searches bean class for all @Inject and create Injectors
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.4 $
 */
public class JndiInjectHandler
{
   public static List loadInjectors(Container container) throws Exception
   {
      Class clazz = container.getBeanClass();
      Method[] methods = clazz.getMethods();
      ArrayList list = new ArrayList();
      InitialContext ctx = container.getInitialContext();
      for (int i = 0; i < methods.length; i++)
      {
         JndiInject ref = (JndiInject) ((Advisor) container).resolveAnnotation(methods[i], JndiInject.class);
         if (ref != null)
         {
            if (!methods[i].getName().startsWith("set")) throw new RuntimeException("@Resource can only be used with a set method: " + methods[i]);
            if (methods[i].getParameterTypes().length != 1) throw new RuntimeException("@Resource can only be used with a set method of one parameter: " + methods[i]);
            String jndiName = methods[i].getName().substring(3);
            if (ref.jndiName() != null && !ref.jndiName().equals(""))
            {
               jndiName = ref.jndiName();
            }
            list.add(new JndiMethodInjector(methods[i], jndiName, ctx));
         }
      }
      loadFieldInjectors(clazz, container, list);
      return list;
   }

   private static void loadFieldInjectors(Class clazz, Container container, ArrayList list) throws Exception
   {
      if (clazz == null || clazz.equals(Object.class)) return;
      loadFieldInjectors(clazz.getSuperclass(), container, list);
      Field[] fields = clazz.getDeclaredFields();
      InitialContext ctx = container.getInitialContext();
      for (int i = 0; i < fields.length; i++)
      {
         JndiInject ref = (JndiInject) ((Advisor) container).resolveAnnotation(fields[i], JndiInject.class);
         if (ref != null)
         {
            String jndiName = fields[i].getName();
            if (ref.jndiName() != null && !ref.jndiName().equals(""))
            {
               jndiName = ref.jndiName();
            }
            list.add(new JndiFieldInjector(fields[i], jndiName, ctx));
         }
      }
   }
}
