/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.injection;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import javax.ejb.TransactionManagementType;
import javax.transaction.UserTransaction;
import org.jboss.aop.Advisor;
import org.jboss.ejb3.BeanContext;
import org.jboss.ejb3.Container;
import org.jboss.ejb3.tx.TxUtil;
import org.jboss.ejb3.tx.UserTransactionImpl;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.2.5 $
 */
public class UserTransactionMethodInjector implements Injector
{
   private Method setMethod;

   public UserTransactionMethodInjector(Method setMethod, Container container)
   {
      TransactionManagementType type = TxUtil.getTransactionManagementType(((Advisor) container));
      if (type != TransactionManagementType.BEAN) throw new IllegalStateException("Container " + container.getEjbName() + ": it is illegal to inject UserTransaction into a CMT bean");
      this.setMethod = setMethod;
   }

   public void inject(BeanContext ctx)
   {
      UserTransaction ut = new UserTransactionImpl();
      Object[] args = {ut};
      try
      {
         setMethod.invoke(ctx.getInstance(), args);
      }
      catch (IllegalAccessException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
      catch (IllegalArgumentException e)
      {
         throw new RuntimeException("Failed in setting EntityManager on setter method: " + setMethod.toString());
      }
      catch (InvocationTargetException e)
      {
         throw new RuntimeException(e.getCause());  //To change body of catch statement use Options | File Templates.
      }
   }
}
