/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.injection.pojo;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Hashtable;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Transient;
import org.jboss.ejb3.EJB3Util;
import org.jboss.ejb3.Ejb3Deployment;
import org.jboss.ejb3.entity.InjectedEntityManager;
import org.jboss.ejb3.entity.InjectedHibernateSession;
import org.jboss.ejb3.entity.ManagedEntityManagerFactory;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.2.3 $
 */
public class PersistenceContextInjector implements PojoInjector
{
   private Hashtable initialContextProperties;

   public PersistenceContextInjector(Hashtable initialContextProperties)
   {
      this.initialContextProperties = initialContextProperties;
   }

   protected Object getEntityManager(boolean isTransient, Class emType, PersistenceContextType type, ManagedEntityManagerFactory factory)
   {
      Object entityManager;
      if (type == PersistenceContextType.EXTENDED)
      {
         throw new RuntimeException("Extended persistence contexts can only be injected in stateful beans");
      }
      else
      {
         if (emType.getName().equals(org.hibernate.Session.class.getName()))
         {
            entityManager = new InjectedHibernateSession(factory, initialContextProperties);
         }
         else
         {
            entityManager = new InjectedEntityManager(factory, initialContextProperties);
         }
      }
      return entityManager;
   }

   protected ManagedEntityManagerFactory getManagedEntityManagerFactory(PersistenceContext ref)
           throws NamingException
   {
      ManagedEntityManagerFactory factory;
      if (ref.unitName() == null || ref.unitName().equals(""))
      {
         throw new RuntimeException("You must specify a unit name for @PersistenceContext");
      }
      else
      {
         InitialContext ctx = EJB3Util.getInitialContext(initialContextProperties);
         factory = (ManagedEntityManagerFactory) ctx.lookup(Ejb3Deployment.MANAGED_ENTITY_FACTORY_CONTEXT + "/" + ref.unitName());
      }
      return factory;
   }

   public void inject(Method m, Annotation annotation, Object pojo)
   {
      InjectionHelper.checkMethodSignature(m, annotation);
      try
      {
         boolean isTransient = m.isAnnotationPresent(Transient.class);
         PersistenceContext pc = m.getAnnotation(PersistenceContext.class);
         ManagedEntityManagerFactory factory = getManagedEntityManagerFactory(pc);
         Object entityManager = getEntityManager(isTransient, m.getParameterTypes()[0], pc.type(), factory);

         m.invoke(pojo, entityManager);
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);
      }
   }

   public void inject(Field f, Annotation annotation, Object pojo)
   {
      try
      {
         boolean isTransient = f.isAnnotationPresent(Transient.class);
         PersistenceContext pc = f.getAnnotation(PersistenceContext.class);
         ManagedEntityManagerFactory factory = getManagedEntityManagerFactory(pc);
         Object entityManager = getEntityManager(isTransient, f.getType(), pc.type(), factory);
         f.setAccessible(true);
         f.set(pojo, entityManager);
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);
      }
   }

}
