/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.security;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

import java.security.Principal;

import javax.annotation.security.RunAs;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.logging.Logger;
import org.jboss.security.AuthenticationManager;
import org.jboss.security.RealmMapping;
import org.jboss.security.RunAsIdentity;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SimplePrincipal;

/**
 * An interceptor that enforces the run-as identity declared by a bean.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>.
 * @version $Revision: 1.3.4.4 $
 */
public class RunAsSecurityInterceptor extends org.jboss.aspects.security.RunAsSecurityInterceptor
{
   private static final Logger log = Logger.getLogger(RunAsSecurityInterceptor.class);
   
   public RunAsSecurityInterceptor(AuthenticationManager manager, RealmMapping realmMapping)
   {
      super(manager, realmMapping);
   }
   
   protected RunAsIdentity getRunAsIdentity(Invocation invocation)
   {
      RunAsIdentity runAsRole = null;
      RunAs runAs = (RunAs) invocation.resolveClassAnnotation(RunAs.class);
      if (runAs == null) return null;
      Principal principal = SecurityAssociation.getPrincipal();
      Set extraRoles = realmMapping.getUserRoles(principal);
      if (extraRoles != null)
      {
         Iterator roles = extraRoles.iterator();
         Set extraRoleNames = new HashSet();
         while (roles.hasNext())
         {
            extraRoleNames.add( ((SimplePrincipal)roles.next()).getName());
         }
         runAsRole = new RunAsIdentity(runAs.value(), principal.getName(), extraRoleNames);
      } else
      {
         runAsRole = new RunAsIdentity(runAs.value(), null);
      }
      
      return runAsRole;
   }
   
   public Object invoke(Invocation invocation) throws Throwable
   {
      try 
      {
         return super.invoke(invocation);
      } finally {
      }
   }

}
