/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.stateful;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.rmi.MarshalledObject;
import java.util.Hashtable;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.jboss.ejb3.BaseSessionContext;
import org.jboss.ejb3.Container;
import org.jboss.ejb3.EJB3Util;
import org.jboss.ejb3.SessionContainer;
import org.jboss.aop.metadata.SimpleMetaData;
import org.jboss.aop.util.PayloadKey;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.2.4 $
 */
public class StatefulBeanContext extends BaseSessionContext implements Externalizable
{
   private static final String BEAN = "STATEFUL_BEAN_INSTANCE";
   private Object id;
   private boolean txSynchronized = false;
   private boolean inInvocation = false;
   private String containerName;
   private MarshalledObject mo;
   private Hashtable initialContextProperties;

   public StatefulBeanContext()
   {
   }

   public boolean isInInvocation()
   {
      return inInvocation;
   }

   public void setInInvocation(boolean inInvocation)
   {
      this.inInvocation = inInvocation;
   }

   public Object getId()
   {
      return id;
   }

   public void setId(Object id)
   {
      this.id = id;
   }

   public boolean isTxSynchronized()
   {
      return txSynchronized;
   }

   public void setTxSynchronized(boolean txSynchronized)
   {
      this.txSynchronized = txSynchronized;
   }

   public void remove()
   {
   }

   public void setContainer(Container container)
   {
      containerName = container.getEjbName();
      initialContextProperties = container.getInitialContextProperties();
      super.setContainer(container);
   }

   public Container getContainer()
   {
      if (container == null)
      {
         SessionContainer result;
         try
         {
            InitialContext ctx = EJB3Util.getInitialContext(initialContextProperties);
            result = (SessionContainer) SessionContainer.getContainersContext(ctx).lookup(containerName);
         }
         catch (NamingException e)
         {
            throw new RuntimeException(e);
         }
         Container container = result;
         setContainer(container);
      }
      return container;
   }

   @Override
   public Object getInstance()
   {
      try
      {
         if (bean == null)
         {
            metadata = (SimpleMetaData)mo.get();
            bean = metadata.getMetaData(BEAN, BEAN);
            getContainer().invokePostActivate(bean);
            mo = null;
         }
      }
      catch (IOException e)
      {
         throw new RuntimeException(e);
      }
      catch (ClassNotFoundException e)
      {
         throw new RuntimeException(e);
      }
      return bean;
   }

   @Override
   public SimpleMetaData getMetaData()
   {
      if (bean == null)
      {
         getInstance(); // get metadata from MO.
      }
      return super.getMetaData();
   }

   // these are publice for fast concurrent access/update
   public volatile boolean markedForPassivation = false;
   public volatile boolean inUse = false;
   public volatile long lastUsed = System.currentTimeMillis();

   public void writeExternal(ObjectOutput out) throws IOException
   {
      out.writeUTF(containerName);
      out.writeObject(id);
      if (mo != null)
         out.writeObject(mo);
      else
      {
         getMetaData().addMetaData(BEAN, BEAN, bean, PayloadKey.AS_IS);
         out.writeObject(new MarshalledObject(getMetaData()));
      }
      out.writeObject(initialContextProperties);
   }

   public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
   {
      containerName = in.readUTF();
      id = in.readObject();
      mo = (MarshalledObject) in.readObject();
      initialContextProperties = (Hashtable)in.readObject();

   }
}
