/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.asynchronous;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import javax.annotation.Resource;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.Local;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContext;

/**
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.3.2.4 $
 */
@Stateless
@Remote(TxSessionRemote.class)
@Local(TxSessionLocal.class)
public class TxSessionBean implements TxSessionRemote, TxSessionLocal
{
   @PersistenceContext EntityManager manager;
   @Resource javax.ejb.SessionContext ctx;

   @TransactionAttribute(TransactionAttributeType.REQUIRED)
   public void createFruit(String name, boolean rollback)
   {
      Fruit fruit = new Fruit(name);
      manager.persist(fruit);
      if (rollback)ctx.setRollbackOnly();
   }

   @TransactionAttribute(TransactionAttributeType.REQUIRED)
   public void createVeg(String name, boolean rollback)
   {
      Vegetable veg = new Vegetable(name);
      manager.persist(veg);
      if (rollback)ctx.setRollbackOnly();
   }

   @TransactionAttribute(TransactionAttributeType.REQUIRED)
   public Collection getEntries()
   {
      Query vegQuery = manager.createQuery("SELECT v FROM Vegetable v");
      List veg = vegQuery.getResultList();
      Query fruitQuery = manager.createQuery("SELECT f FROM Fruit f");
      List fruit = fruitQuery.getResultList();


      ArrayList list = new ArrayList();
      for (Iterator it = veg.iterator() ; it.hasNext() ; )
      {
         list.add(it.next().toString());
      }

      for (Iterator it = fruit.iterator() ; it.hasNext() ; )
      {
         list.add(it.next().toString());
      }


      return list;
   }

   @TransactionAttribute(TransactionAttributeType.REQUIRED)
   public void cleanAll()
   {
      Query vegQuery = manager.createQuery("SELECT v FROM Vegetable v");
      List veg = vegQuery.getResultList();
      for (Iterator it = veg.iterator() ; it.hasNext() ; )
      {
         manager.remove(it.next());
      }

      Query fruitQuery = manager.createQuery("SELECT f FROM Fruit f");
      List fruit = fruitQuery.getResultList();
      for (Iterator it = fruit.iterator() ; it.hasNext() ; )
      {
         manager.remove(it.next());
      }
   }

}
