/*
 *
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.ejb3.test.cache.unit;

//import java.rmi.*;


import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import org.jboss.ejb3.test.cache.SimpleStatefulRemote;
import org.jboss.ejb3.test.cache.StatefulRemote;
import org.jboss.test.JBossTestCase;
import junit.framework.Test;

/**
 * Sample client for the jboss container.
 *
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Id: StatefulUnitTestCase.java,v 1.2.6.4 2005/09/14 22:19:02 bill Exp $
 */

public class StatefulUnitTestCase
extends JBossTestCase
{
   org.apache.log4j.Category log = getLog();

   static boolean deployed = false;
   static int test = 0;

   public StatefulUnitTestCase(String name)
   {

      super(name);

   }

   public void testStateful() throws Exception
   {
      StatefulRemote remote = (StatefulRemote) getInitialContext().lookup(StatefulRemote.class.getName());
      remote.reset();
      remote.setState("hello");
      Thread.sleep(5000);
      assertEquals("hello", remote.getState());
      assertTrue(remote.getPostActivate());
      assertTrue(remote.getPrePassivate());
   }

   public void testStatefulLongRunning() throws Exception
   {
      StatefulRemote remote = (StatefulRemote) getInitialContext().lookup(StatefulRemote.class.getName());
      remote.reset();
      remote.setState("hello");
      remote.longRunning();
      assertEquals("hello", remote.getState());
      assertTrue(remote.getPostActivate());
      assertTrue(remote.getPrePassivate());
   }

   public void testSimpleStatefulLongRunning() throws Exception
   {
      SimpleStatefulRemote remote = (SimpleStatefulRemote) getInitialContext().lookup(SimpleStatefulRemote.class.getName());
      remote.reset();
      remote.setState("hello");
      remote.longRunning();
      assertEquals("hello", remote.getState());
      assertTrue(remote.getPostActivate());
      assertTrue(remote.getPrePassivate());
   }

   public void testSimpleStateful() throws Exception
   {
      SimpleStatefulRemote remote = (SimpleStatefulRemote) getInitialContext().lookup(SimpleStatefulRemote.class.getName());
      remote.reset();
      remote.setState("hello");
      Thread.sleep(5000);
      assertEquals("hello", remote.getState());
      assertTrue(remote.getPostActivate());
      assertTrue(remote.getPrePassivate());
   }

   public void testSimpleLocal() throws Exception
   {
	   MBeanServerConnection server = getServer();
      ObjectName testerName = new ObjectName("jboss.ejb3:service=Tester,test=cache");
      Object[] params = {};
      String[] sig = {};
      server.invoke(testerName, "testSimpleLocal", params, sig);
   }

   public void testBench() throws Exception
   {
      StatefulRemote remote = (StatefulRemote) getInitialContext().lookup(StatefulRemote.class.getName());
      System.out.println("bench: " + remote.bench(1000));

   }

   public void testBenchSimple() throws Exception
   {
      SimpleStatefulRemote remote = (SimpleStatefulRemote) getInitialContext().lookup(SimpleStatefulRemote.class.getName());
      System.out.println("simple bench: " + remote.bench(1000));

   }

   public static Test suite() throws Exception
   {
      return getDeploySetup(StatefulUnitTestCase.class, "testejb3-statefulcache-service.xml, cache-test.ejb3, cache-test.sar");
   }

}
