/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.consumer;

import org.jboss.annotation.JndiInject;
import javax.ejb.Stateless;
import javax.ejb.Remote;
import org.jboss.ejb3.mdb.ProducerManager;
import org.jboss.ejb3.mdb.ProducerObject;
import org.jboss.annotation.JndiInject;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.5.2.4 $
 */
@Stateless
@Remote(TestStatus.class)
public class TestStatusBean implements TestStatus
{
   public static String queueRan = null;
   public static String topicRan = null;
   public static String interceptedTopic = null;
   public static String interceptedQueue = null;
   public static boolean postConstruct = false;
   public static boolean preDestroy = false;

   public void clear()
   {
      queueRan = null;
      topicRan = null;
      interceptedTopic = null;
      interceptedQueue = null;
      postConstruct = false;
      preDestroy = false;
   }

   private QueueTestXA xa;
   private ProducerManager xaManager;

   @JndiInject(jndiName="org.jboss.ejb3.test.consumer.QueueTestXA") public void setXa(QueueTestXA xa)
   {
      this.xa = xa;
      this.xaManager = ((ProducerObject)xa).getProducerManager();
   }

   private QueueTest local;
   private ProducerManager localManager;

   @JndiInject(jndiName="org.jboss.ejb3.test.consumer.QueueTestLocal") public void setLocal(QueueTest local)
   {
      this.local = local;
      this.localManager = ((ProducerObject)local).getProducerManager();
   }

   public void testXA() throws Exception
   {

      xaManager.connect();
      xa.method1("testXA", 1);
      xa.method2("testXA2", 4.4F);
      System.out.println("end TESTXA **");
      xaManager.close();
   }

   public void testLocal() throws Exception
   {

      localManager.connect();
      local.method1("testLocal", 1);
      local.method2("testLocal2", 4.4F);
      localManager.close();
   }

   public String queueFired()
   {
      return queueRan;
   }

   public String topicFired()
   {
      return topicRan;
   }

   public String interceptedTopic()
   {
      return interceptedTopic;
   }

   public String interceptedQueue()
   {
      return interceptedQueue;
   }

   public boolean postConstruct()
   {
      return postConstruct;
   }

   public boolean preDestroy()
   {
      return preDestroy;
   }
}
