/*
* JBoss, Home of Professional Open Source
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.ejb3.test.dd.web.ejb;

import javax.naming.InitialContext;
import javax.annotation.security.PermitAll;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionContext;

import org.jboss.ejb3.test.dd.web.interfaces.ReferenceTest;
import org.jboss.ejb3.test.dd.web.interfaces.StatelessSession;
import org.jboss.ejb3.test.dd.web.interfaces.ReturnData;
import org.jboss.logging.Logger;

/** A stateless SessionBean 

 @author  Scott.Stark@jboss.org
 @version $Revision: 1.2.2.4 $
 */
public class StatelessSessionBean2
{
   static Logger log = Logger.getLogger(StatelessSessionBean2.class);

   private SessionContext sessionContext;

   public void setSessionContext(SessionContext context)
   {
      sessionContext = context;
   }

   public String echo(String arg)
   {
      return arg;
   }

   public String forward(String echoArg)
   {
      String echo = null;
      try
      {
         InitialContext ctx = new InitialContext();
         StatelessSession bean = (StatelessSession) ctx.lookup("java:comp/env/ejb/Session");
         echo = bean.echo(echoArg);
      }
      catch (Exception e)
      {
         throw new EJBException(e);
      }
      return echo;
   }

   public void noop(ReferenceTest test, boolean optimized)
   {
      boolean wasSerialized = test.getWasSerialized();
    
      if (optimized && wasSerialized == true)
         throw new EJBException("Optimized call had non-optimized(i.e. serialized) argument");
      if (optimized == false && wasSerialized == false)
         throw new EJBException("Non-optimized call had optimized (i.e. non-serialized) argument");
   }

   public ReturnData getData()
   {
      ReturnData data = new ReturnData();
      data.data = "TheReturnData2";
      return data;
   }

   /** A method deployed with no method permissions */
   public void unchecked()
   {
      log.debug("unchecked");
   }

   /** A method deployed with method permissions such that only a run-as
    * assignment will allow access. 
    */
   public void checkRunAs()
   {
      log.debug("checkRunAs");
   }
}
