/*
* JBoss, Home of Professional Open Source
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.ejb3.test.dd.web.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.UserTransaction;

import org.jboss.ejb3.Container;
import org.jboss.ejb3.Ejb3Deployment;
import org.jboss.ejb3.entity.ManagedEntityManagerFactory;
import org.jboss.ejb3.test.dd.web.ejb.Address;
import org.jboss.ejb3.test.dd.web.interfaces.ReferenceTest;
import org.jboss.ejb3.test.dd.web.interfaces.Session30;
import org.jboss.ejb3.test.dd.web.interfaces.StatelessSession;
import org.jboss.ejb3.test.dd.web.interfaces.StatelessSessionLocal;
import org.jboss.ejb3.test.dd.web.util.Util;
import org.jboss.logging.Logger;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SimplePrincipal;

/** A servlet that accesses an EJB and tests whether the call argument
 is serialized.

 @author  Scott.Stark@jboss.org
 @version $Revision: 1.2.2.4 $
 */
public class EJBServlet extends HttpServlet
{
   private static final Logger log = Logger.getLogger(EJBServlet.class);
   
   public void init(ServletConfig config) throws ServletException
   {
       String param = config.getInitParameter("failOnError");
       boolean failOnError = true;
       if( param != null && Boolean.valueOf(param).booleanValue() == false )
           failOnError = false;
       try
       {
           // Access the Util.configureLog4j() method to test classpath resource
           URL propsURL = Util.configureLog4j();
           log.debug("log4j.properties = "+propsURL);
       }
       catch(Exception e)
       {
           log.debug("failed", e);
           ClassLoader loader = Thread.currentThread().getContextClassLoader();
           try
           {
              log.debug(Util.displayClassLoaders(loader));
           }
           catch(NamingException ne)
           {
              log.debug("failed", ne);
           }
           if( failOnError == true )
               throw new ServletException("Failed to init EJBOnStartupServlet", e);
       }
   }
   
   protected void processRequest(HttpServletRequest request, HttpServletResponse response)
         throws ServletException, IOException
   {
      UserTransaction tx = null;
      try
      {
         SecurityAssociation.setPrincipal(new SimplePrincipal("jduke"));
         SecurityAssociation.setCredential("theduke".toCharArray());
         
         InitialContext ctx = new InitialContext();
        
         StatelessSession optimizedBean = (StatelessSession)ctx.lookup("java:comp/env/ejb/OptimizedEJB");
         optimizedBean.noop(new ReferenceTest(), false);
         
         StatelessSessionLocal localOptimizedBean = (StatelessSessionLocal)ctx.lookup("java:comp/env/ejb/local/OptimizedEJB");
         localOptimizedBean.noop(new ReferenceTest(), true);
         
         Session30 session30 = (Session30)ctx.lookup("java:comp/env/ejb/Session30");
         String access = session30.access();
               
         ManagedEntityManagerFactory managerFactory = (ManagedEntityManagerFactory)ctx.lookup(Ejb3Deployment.MANAGED_ENTITY_FACTORY_CONTEXT + "/EntityManager");
         EntityManager entityManager = managerFactory.getEntityManagerFactory().createEntityManager();
         
         Address address = new Address();
         address.setStreet("Clarendon Street");
         address.setCity("Boston");
         address.setState("MA");
         address.setZip("02116");
         
         tx = (UserTransaction)ctx.lookup("UserTransaction");
         tx.begin();
         entityManager.persist(address);
         tx.commit();
      }
      catch (Exception e)
      {
         if (tx != null)
         {
            try {
               tx.rollback();
            } catch (Exception e1){
              
            }
         }
         throw new ServletException("Failed to call OptimizedEJB/Session30 through remote and local interfaces", e);
      }
      response.setContentType("text/html");
      PrintWriter out = response.getWriter();
      out.println("<html>");
      out.println("<head><title>EJBServlet</title></head>");
      out.println("<body>Tests passed<br>Time:" + Util.getTime() + "</body>");
      out.println("</html>");
      out.close();
   }

   protected void doGet(HttpServletRequest request, HttpServletResponse response)
         throws ServletException, IOException
   {
      processRequest(request, response);
   }

   protected void doPost(HttpServletRequest request, HttpServletResponse response)
         throws ServletException, IOException
   {
      processRequest(request, response);
   }
}
