/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.factory;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceUnit;
import org.hibernate.SessionFactory;
import org.hibernate.Session;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.4 $
 */
@Stateless
@Remote(Session1.class)
public class Session1Bean implements Session1
{
   @PersistenceContext Session manager1;
   @PersistenceContext(unitName="manager2") EntityManager manager2;
   @PersistenceUnit SessionFactory factory1;
   @PersistenceUnit(unitName="manager2") EntityManagerFactory factory2;

   public int create1FromManager()
   {
      Entity1 one = new Entity1();
      one.setString("oneManager");
      manager1.save(one);
      return one.getId();
   }
   public int create1FromFactory()
   {
      Entity1 one = new Entity1();
      Session m = factory1.openSession();
      m.save(one);
      m.close();
      return one.getId();
   }

   public int create2FromManager()
   {
      Entity2 two = new Entity2();
      two.setString("twoManager");
      manager2.persist(two);
      return two.getId();
   }
   public int create2FromFactory()
   {
      Entity2 two = new Entity2();
      EntityManager m = factory2.createEntityManager();
      m.persist(two);
      m.close();
      return two.getId();
   }

   public void doUtil(Util ux)
   {
      Util u = new Util();
      u.setId(1);
      u.setName("one");
      manager1.persist(u);

      u = new Util();
      u.setId(2);
      u.setName("two");
      manager2.persist(u);
   }

}
