/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.factory;

import javax.annotation.Resource;
import javax.ejb.Remote;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceUnit;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.2.4 $
 */
@Stateless
@Remote(Session2.class)
public class Session2Bean implements Session2
{
   @PersistenceContext EntityManager manager2;
   @PersistenceContext(unitName="manager1") EntityManager manager1;
   @PersistenceUnit EntityManagerFactory factory2;
   @PersistenceUnit(unitName="manager1") EntityManagerFactory factory1;

   @Resource SessionContext ctx;

   public Entity1 find1FromManager(int id)
   {
      EntityManager m1 = (EntityManager)ctx.lookup("java:/EntityManagers/manager1");
      if (m1.find(Entity1.class, id) == null) throw new RuntimeException("failed to find");
      return manager1.find(Entity1.class, id);
   }
   public void find1FromFactory(int id)
   {
      EntityManagerFactory f1 = (EntityManagerFactory)ctx.lookup("java:/EntityManagerFactories/manager1");
      EntityManager m1 = f1.getEntityManager();
      if (m1.find(Entity1.class, id) == null) throw new RuntimeException("failed to find");

      EntityManager m = factory1.getEntityManager();
      Entity1 one = m.find(Entity1.class, id);
      if (one == null) throw new RuntimeException("NOT FOUND");

   }
   public Entity2 find2FromManager(int id)
   {
      return manager2.find(Entity2.class, id);
   }
   public void find2FromFactory(int id)
   {
      EntityManager m = factory2.createEntityManager();
      Entity2 two = m.find(Entity2.class, id);
      if (two == null) throw new RuntimeException("NOT FOUND");
      m.close();
   }

   public Util findUtil1FromManager(int id)
   {
      return manager1.find(Util.class, id);
   }

   public Util findUtil2FromManager(int id)
   {
      return manager2.find(Util.class, id);
   }
}
