/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.initial;

import javax.ejb.AroundInvoke;
import javax.ejb.Interceptors;
import javax.ejb.InvocationContext;
import javax.ejb.PostActivate;
import javax.ejb.PostConstruct;
import javax.ejb.PreDestroy;
import javax.ejb.PrePassivate;
import javax.ejb.Stateless;

/**
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.6.2.4 $
 */
@Stateless
@Interceptors ({FirstInterceptor.class, SecondInterceptor.class})
public class InterceptedSLTestBean implements InterceptedSLTest
{
   public int testMethod(int i)
   {
      System.out.println("InterceptedSLTestBean testMethod");
      return i;
   }

   public int throwsThrowable(int i)throws Throwable
   {
      System.out.println("InterceptedSLTestBean throwsThrowable");
      throw new Throwable();
   }

   @AroundInvoke
   public Object myInterceptor(InvocationContext ctx) throws Exception
   {
      System.out.println("Intercepting in InterceptedSLTestBean.myInterceptor()");
      int val = (Integer)ctx.getContextData().get("DATA");

      int ret = (Integer)ctx.proceed();
      ctx.getEJBContext().setRollbackOnly();
      val = (Integer)ctx.getContextData().get("DATA");
      ret += val;
      if (ctx.getBean() != this) throw new RuntimeException("ctx.getBean() != this: " + ctx.getBean() + " != " + this);
      return ret;
   }

   @PostConstruct
   public void postConstruct()
   {
      TestStatusBean.postConstruct = true;
   }

   @PreDestroy
   public void preDestroy()
   {
      TestStatusBean.preDestroy = true;
   }

   @PrePassivate
   public void prePassivate()
   {
      throw new RuntimeException("SLSBs don't have PrePassivate methods");
   }

   @PostActivate
   public void postActivate()
   {
      throw new RuntimeException("SLSBs don't have PostActivate methods");
   }

}
