/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.initial;

import java.util.Map;
import javax.naming.InitialContext;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3.2.5 $
 */
public class Tester implements TesterMBean
{
   public void testSLSBCollocation() throws Exception
   {
      InitialContext ctx = new InitialContext();
      TestLocal local = (TestLocal) ctx.lookup(TestLocal.class.getName());
      TestRemote remote = (TestRemote) ctx.lookup(TestRemote.class.getName());

      if (local.getObject() != TestBean.obj) throw new RuntimeException("Local call not equal");
      if (local.getObject() == remote.getObject()) throw new RuntimeException("Remote should not be equal");
      Map map = remote.getObject();
      map.put("hello", "world");

      Object obj = local.echo(map);
      if (obj != map) throw new RuntimeException("argument and return should be the same");

      map = (Map)obj;
      if (!map.containsKey("hello")) throw new RuntimeException("not good return");

      obj = remote.echo(map);
      if (obj == map) throw new RuntimeException("argument and return should not be the same");

      map = (Map)obj;
      if (!map.containsKey("hello")) throw new RuntimeException("not good return");

   }

   public void testSFSBCollocation() throws Exception
   {
      InitialContext ctx = new InitialContext();
      StatefulTestLocal local = (StatefulTestLocal) ctx.lookup(StatefulTestLocal.class.getName());
      StatefulTestRemote remote = (StatefulTestRemote) ctx.lookup(StatefulTestRemote.class.getName());

      if (local.getObject() != StatefulTestBean.obj) throw new RuntimeException("Local call not equal");
      if (local.getObject() == remote.getObject()) throw new RuntimeException("Remote should not be equal");
      Map map = remote.getObject();
      map.put("hello", "world");

      Object obj = local.echo(map);
      if (obj != map) throw new RuntimeException("argument and return should be the same");
      map = (Map)obj;
      if (!map.containsKey("hello")) throw new RuntimeException("not good return");

      obj = remote.echo(map);
      if (obj == map) throw new RuntimeException("argument and return should not be the same");

      map = (Map)obj;
      if (!map.containsKey("hello")) throw new RuntimeException("not good return");
   }


   public void test() throws Exception
   {
      InitialContext ctx = new InitialContext();
      Test test = (Test) ctx.lookup(TestLocal.class.getName());
      String echo = test.testMe("echo");
      if (!"echo".equals(echo)) throw new RuntimeException("ECHO FAILED!");
   }

   public void statefulTest() throws Exception
   {
      InitialContext ctx = new InitialContext();
      StatefulTestLocal test = (StatefulTestLocal) ctx.lookup(StatefulTestLocal.class.getName());
      test.setState("hello world");
      if (!test.getState().equals("hello world")) throw new Exception("state was not retained");
   }

   public void testInterceptors() throws Exception
   {
      InitialContext ctx = new InitialContext();
      InterceptedSLTest test = (InterceptedSLTest) ctx.lookup(InterceptedSLTest.class.getName());
      int ret = test.testMethod(5);
      int expected = 3010;

      if (ret != expected) throw new Exception("return value was not " + expected + ", it was: " + ret);

      boolean exception = false;
      try
      {
         test.throwsThrowable(1);
      }
      catch (Throwable e)
      {
         exception = true;
      }
      if (!exception) throw new Exception("Exception was not thrown");
   }

   public void testCallbacks() throws Exception
   {
      //Check the correct callbacks get invoked
      InitialContext ctx = new InitialContext();
      TestStatus status = (TestStatus) ctx.lookup(TestStatus.class.getName());
      status.clear();
      InterceptedSLTest test = (InterceptedSLTest) ctx.lookup(InterceptedSLTest.class.getName());
      test.testMethod(5);
      if (!status.postConstruct()) throw new Exception("PostConstruct should be called for SLSB");
      if (status.prePassivate()) throw new Exception("PrePassivate should not be called for SLSB");
      if (status.postActivate()) throw new Exception("PostActivate should not be called for SLSB");

      //TODO: Figure out when destroy gets called
      //if (!status.preDestroy()) throw new Exception("PreDestroy should be called for SLSB");
   }
}



