/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.initial;

import org.jboss.tm.TransactionManagerService;

import javax.naming.InitialContext;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3.2.5 $
 */
public class TxTester implements TxTesterMBean
{
   public void testTransactions() throws Exception
   {
      InitialContext ctx = new InitialContext();
      TransactionManager tm = (TransactionManager) ctx.lookup(TransactionManagerService.JNDI_NAME);
      TestLocal test = (TestLocal) ctx.lookup(TestLocal.class.getName());
      callNever(tm, test);
      callNotSupported(tm, test);
      callSupportsWithTx(tm, test);
      callSupportsWithoutTx(tm, test);
      test.required();
      callMandatoryNoTx(tm, test);
      callMandatoryWithTx(tm, test);
      callRequiresNew(tm, test);
   }

   public void callRequiresNew(TransactionManager tm, TestLocal test) throws Exception
   {
      tm.begin();
      Transaction tx = tm.getTransaction();
      test.requiresNew(tx);
      tm.commit();
   }

   public void callNever(TransactionManager tm, TestLocal test) throws Exception
   {
      boolean exceptionThrown = false;
      tm.begin();
      try
      {
         test.never();
      }
      catch (Exception ex)
      {
         //System.out.println("**************");
         //ex.printStackTrace();
         //System.out.println("******************");
         exceptionThrown = true;
      }
      tm.rollback();
      if (!exceptionThrown) throw new Exception("failed on mandatory no tx call");
   }

   public void callNotSupported(TransactionManager tm, TestLocal test) throws Exception
   {
      tm.begin();
      test.notSupported();
      tm.commit();
   }

   public void callSupportsWithTx(TransactionManager tm, TestLocal test) throws Exception
   {
      tm.begin();
      Transaction tx = tm.getTransaction();
      test.supports(tx);
      tm.commit();
   }

   public void callSupportsWithoutTx(TransactionManager tm, TestLocal test) throws Exception
   {
      test.supports(null);
   }

   public void callMandatoryNoTx(TransactionManager tm, TestLocal test) throws Exception
   {
      boolean exceptionThrown = false;
      try
      {
         test.mandatory();
      }
      catch (Exception ex)
      {
         exceptionThrown = true;
      }
      if (!exceptionThrown) throw new Exception("failed on mandatory no tx call");
   }

   public void callMandatoryWithTx(TransactionManager tm, Test test) throws Exception
   {
      tm.begin();
      test.mandatory();
      tm.commit();
   }
}



