/*
 * JBoss, the open source webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.ejb3.test.localcall.unit;

import javax.naming.InitialContext;

import junit.framework.Test;

import org.jboss.ejb3.test.localcall.ServiceRemote;
import org.jboss.ejb3.test.localcall.StatefulClustered;
import org.jboss.ejb3.test.localcall.StatefulRemote;
import org.jboss.ejb3.test.localcall.StatelessClustered;
import org.jboss.ejb3.test.localcall.StatelessRemote;
import org.jboss.test.JBossTestCase;

/**
 *
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.1.2.5 $
 */
public class TestLocalCallsTestCase extends JBossTestCase
{
   public TestLocalCallsTestCase(String name)
   {
      super(name);
      
   }
   
   public void testRemoteIfLocalCalls() throws Exception
   {
      System.out.println("Testing local");
      InitialContext ctx = getInitialContext();
      StatefulRemote stateful1 = (StatefulRemote)ctx.lookup(StatefulRemote.class.getName());
      StatefulRemote stateful2 = (StatefulRemote)ctx.lookup(StatefulRemote.class.getName());
      
      StatefulClustered statefulClustered1 = (StatefulClustered)ctx.lookup(StatefulClustered.class.getName());
      StatefulClustered statefulClustered2 = (StatefulClustered)ctx.lookup(StatefulClustered.class.getName());
      
      StatelessRemote stateless1 = (StatelessRemote)ctx.lookup(StatelessRemote.class.getName());
      StatelessRemote stateless2 = (StatelessRemote)ctx.lookup(StatelessRemote.class.getName());
      
      StatelessClustered statelessClustered1 = (StatelessClustered)ctx.lookup(StatelessClustered.class.getName());
      StatelessClustered statelessClustered2 = (StatelessClustered)ctx.lookup(StatelessClustered.class.getName());
      
      ServiceRemote service1 = (ServiceRemote)ctx.lookup(ServiceRemote.class.getName());
      ServiceRemote service2 = (ServiceRemote)ctx.lookup(ServiceRemote.class.getName());
      
      stateful1.test();
      stateful2.test();
      statefulClustered1.test();
      statefulClustered2.test();
      stateless1.test();
      stateless2.test();
      statelessClustered1.test();
      statelessClustered2.test();
      service1.test();
      service2.test();
            
      assertFalse(stateful1.hashCode() ==  stateful2.hashCode());
      assertFalse(statefulClustered1.hashCode() == statefulClustered2.hashCode());
      assertTrue(stateless1.hashCode() == stateless2.hashCode());
      assertTrue(statelessClustered1.hashCode() == statelessClustered2.hashCode());
      assertTrue(service1.hashCode() == service2.hashCode());
      
      assertFalse(stateful1.equals(stateful2));
      assertFalse(statefulClustered1.equals(statefulClustered2));
      assertTrue(stateless1.equals(stateless2));
      assertTrue(statelessClustered1.equals(statelessClustered2));
      assertTrue(service1.equals(service2));
      
   }
   
   public void testLocalIfLocalCalls() throws Exception
   {
      System.out.println("TESTING LOCAL CALLS");
      InitialContext ctx = getInitialContext();
      ServiceRemote service = (ServiceRemote)ctx.lookup(ServiceRemote.class.getName());
      service.testLocal();
   }
   
   public static Test suite() throws Exception
   {
      return getDeploySetup(TestLocalCallsTestCase.class, "localcall-test.ejb3");
   }
}
