/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.reference21_30;

import javax.naming.*;
import javax.annotation.EJB;
import javax.annotation.EJBs;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.Local;

import org.jboss.annotation.ejb.LocalBinding;
import org.jboss.annotation.ejb.RemoteBinding;
import org.jboss.ejb3.Container;
import org.jboss.logging.Logger;

/**
 * @version <tt>$Revision: 1.2.2.6 $</tt>
 * @author <a href="mailto:bdecoste@jboss.com">William DeCoste</a>
 */
@Stateless(name="Session30")
@Remote(Session30.class)
@Local(LocalSession30.class)
@RemoteBinding(jndiBinding = "Session30")
@LocalBinding(jndiBinding = "LocalSession30")
@EJBs({@EJB(name="injected", businessInterface=org.jboss.ejb3.test.reference21_30.Session21.class, beanName="Session21")})
public class Session30Bean 
{
   private static final Logger log = Logger.getLogger(Session30Bean.class);
   
   public String access()
   {
      return "Session30";
   }
   
   public String access21()
   {
      try {
         InitialContext jndiContext = new InitialContext();
         Session21 session = (Session21)jndiContext.lookup(Container.ENC_CTX_NAME + "/env/ejb/injected");
         return session.access();
      } catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   public String globalAccess21()
   {
      try {
         InitialContext jndiContext = new InitialContext();
         Session21Home home = (Session21Home)jndiContext.lookup("Session21");
         Session21 session = (Session21)home.create();
         return session.access();
      } catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   public String accessLocalStateful()
   {
      try {
         InitialContext jndiContext = new InitialContext();
         StatefulSession30LocalHome localHome = (StatefulSession30LocalHome)jndiContext.lookup("LocalStatefulSession30");
         LocalStatefulSession30 localSession = localHome.create();
         return localSession.getValue();
      } catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   public String accessLocalStateful(String value)
   {
      try {
         InitialContext jndiContext = new InitialContext();
         StatefulSession30LocalHome localHome = (StatefulSession30LocalHome)jndiContext.lookup("LocalStatefulSession30");
         LocalStatefulSession30 localSession = localHome.create(value);
         return localSession.getValue();
      } catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   public String accessLocalStateful(String value, Integer suffix)
   {
      try {
         InitialContext jndiContext = new InitialContext();
         StatefulSession30LocalHome localHome = (StatefulSession30LocalHome)jndiContext.lookup("LocalStatefulSession30");
         LocalStatefulSession30 localSession = localHome.create(value, suffix);
         return localSession.getValue();
      } catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
}
