/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.reference21_30;

import javax.ejb.Init;
import javax.ejb.Local;
import javax.ejb.PreDestroy;
import javax.ejb.Remote;
import javax.ejb.Stateful;
import javax.naming.InitialContext;
import org.jboss.annotation.ejb.LocalBinding;
import org.jboss.annotation.ejb.RemoteBinding;
import org.jboss.logging.Logger;

/**
 * @version <tt>$Revision: 1.4.2.5 $</tt>
 * @author <a href="mailto:bdecoste@jboss.com">William DeCoste</a>
 */
@Stateful(name="StatefulSession30")
@Local(LocalStatefulSession30.class)
@Remote(StatefulSession30.class)
@RemoteBinding(jndiBinding = "StatefulSession30")
@LocalBinding(jndiBinding = "LocalStatefulSession30")
public class StatefulSession30Bean implements java.io.Serializable
{
   private static final Logger log = Logger.getLogger(StatefulSession30Bean.class);
   
   private String value = null;
   
   public void setValue(String value)
   {
      this.value = value;
   }
   
   public String getValue()
   {
      return value;
   }
   
   public String accessLocalStateless()
   {
      try {
         InitialContext jndiContext = new InitialContext();
         Session30LocalHome localHome = (Session30LocalHome)jndiContext.lookup("LocalSession30");
         LocalSession30 localSession = localHome.create();
         return localSession.access();
      } catch (Exception e)
      {
         e.printStackTrace();
         return null;
      }
   }
   
   @Init
   public void ejbCreate()
   {
      value="default";
   }
   
   @Init
   public void ejbCreate(String value)
   {
      this.value=value;
   }
   
   @Init
   public void ejbCreate(String value, Integer suffix)
   {
      this.value=value + suffix;
   }
   
   @PreDestroy
   public void preDestroy()
   {
      log.info("Invoking PreDestroy");
   }
}
