/*
 *
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.ejb3.test.relationships.unit;

//import java.rmi.*;


import java.util.Collection;
import java.util.Iterator;
import org.jboss.ejb3.test.relationships.Customer;
import org.jboss.ejb3.test.relationships.LineItem;
import org.jboss.ejb3.test.relationships.Order;
import org.jboss.ejb3.test.relationships.SessionRemote;
import org.jboss.test.JBossTestCase;
import junit.framework.Test;

/**
 * Test reentrant remote call.  There was a bug that remoting didn't route locally
 * and the tx propagation was happening when it shouldn't
 *
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Id: RelationshipUnitTestCase.java,v 1.2.2.4 2005/09/14 22:19:14 bill Exp $
 */

public class RelationshipUnitTestCase
extends JBossTestCase
{
   org.apache.log4j.Category log = getLog();

   static boolean deployed = false;
   static int test = 0;

   public RelationshipUnitTestCase(String name)
   {

      super(name);

   }

   public void testUniOneToMany() throws Exception
   {
      SessionRemote remote = (SessionRemote) getInitialContext().lookup(SessionRemote.class.getName());
      remote.createCategory();
      remote.createItem();
      int num = remote.getNumItems();
      assertEquals(1, num);
   }

   public void testOneToOne() throws Exception
   {
      SessionRemote remote = (SessionRemote) getInitialContext().lookup(SessionRemote.class.getName());
      long id = remote.createCustomer();
      Customer cust = remote.getCustomer(id);
      long record = cust.getCustomerRecord().getId();
      remote.testInverse(record);
      cust = remote.getCustomer(id);
      record = cust.getCustomerRecord().getId();

   }

   public void testMappedByOneToMany() throws Exception
   {
      SessionRemote remote = (SessionRemote) getInitialContext().lookup(SessionRemote.class.getName());
      long id1 = remote.createCustomer();
      Customer cust1 = remote.getCustomer(id1);
      long orderId = remote.creatOrder();
      Order order = remote.getOrder(orderId);
      order.setCustomer(cust1);
      remote.merge(order);
      orderId = remote.creatOrder();
      order = remote.getOrder(orderId);
      order.setCustomer(cust1);
      remote.merge(order);
      cust1 = remote.getCustomer(id1);
      assertEquals(cust1.getOrders().size(), 2);
      for (Order orderIn : cust1.getOrders())
      {
         orderIn.setCustomer(null);
         remote.merge(orderIn);
         break;
      }
      cust1 = remote.getCustomer(id1);
      assertEquals(cust1.getOrders().size(), 1);



   }

   public void testRemove() throws Exception
   {

      SessionRemote remote = (SessionRemote) getInitialContext().lookup(SessionRemote.class.getName());
      long id = remote.creatOrder();
      Order order = remote.getOrder(id);
      assertEquals(order.getItems().size(), 2);
      remote.deleteFromCollection(id);
      order = remote.getOrder(id);
      assertEquals(order.getItems().size(), 1);
      for (LineItem item : order.getItems())
      {
         remote.deleteOne(item.getId());
         break;
      }
      order = remote.getOrder(id);
      assertEquals(order.getItems().size(), 0);
   }

   public void testMergeRemove() throws Exception
   {

      SessionRemote remote = (SessionRemote) getInitialContext().lookup(SessionRemote.class.getName());
      long id = remote.creatOrder();
      Order order = remote.getOrder(id);
      Collection<LineItem> items = order.getItems();
      Iterator<LineItem> it = items.iterator();
      while (it.hasNext())
      {
         LineItem item = it.next();
         item.setOrder(null);
         break;
      }
      remote.merge(order);
      order = remote.getOrder(id);
      assertEquals(order.getItems().size(), 1);
   }

   public static Test suite() throws Exception
   {
      return getDeploySetup(RelationshipUnitTestCase.class, "relationships-test.ejb3");
   }

}
