/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.service;

import javax.ejb.Remote;
import javax.ejb.Local;
import org.jboss.annotation.ejb.Service;

/**
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.5.2.4 $
 */
@Service
@Remote(ServiceOneRemote.class)
@Local(ServiceOneLocal.class)
public class ServiceOne implements ServiceOneLocal, ServiceOneRemote, ServiceOneManagement
{
   static int instances = 0;
   int localMethodCalls;
   int remoteMethodCalls;
   int jmxAttribute;
   int someJmxAttribute;
   int otherJmxAttribute;
   int readWriteOnlyAttribute;

   public ServiceOne()
   {
      instances++;
   }

   public int getInstances()
   {
      return instances;
   }

   public int getRemoteMethodCalls()
   {
      return remoteMethodCalls;
   }

   public void setRemoteMethodCalls(int i)
   {
      remoteMethodCalls = i;
   }

   public int getLocalMethodCalls()
   {
      return localMethodCalls;
   }

   public void setLocalMethodCalls(int i)
   {
      localMethodCalls = i;
   }

   public void localMethod(String s)
   {
      localMethodCalls++;
   }

   public void remoteMethod(String s)
   {
      remoteMethodCalls++;
   }

   public String jmxOperation(String s)
   {
      return "x" + s + "x";
   }

   public String[] jmxOperation(String[] s)
   {
      for (int i = 0 ; i < s.length ; i++)
      {
         s[i] = jmxOperation(s[i]);
      }
      return s;
   }

   public int getAttribute()
   {
      return jmxAttribute;
   }

   public void setAttribute(int i)
   {
      jmxAttribute = i;
   }

   public int getSomeAttr()
   {
      return someJmxAttribute;
   }

   public void setSomeAttr(int i)
   {
      someJmxAttribute = i;
   }

   public int getOtherAttr()
   {
      return otherJmxAttribute;
   }

   public void setOtherAttr(int i)
   {
      otherJmxAttribute = i;
   }

   public void setWriteOnly(int i)
   {
      readWriteOnlyAttribute = i;
   }

   public int getReadOnly()
   {
      return readWriteOnlyAttribute;
   }


   public void create() throws Exception
   {
      System.out.println("ServiceOne - CREATE");
      Tester.creates.add("1");
   }

   public void start() throws Exception
   {
      System.out.println("ServiceOne - START");
      Tester.starts.add("1");
   }

   public void stop()
   {
      System.out.println("ServiceOne - STOP");
   }

   public void destroy()
   {
      System.out.println("ServiceOne - DESTROY");
   }

}
