/*
 *
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.ejb3.test.service.unit;

//import java.rmi.*;


import junit.framework.Test;
import org.jboss.test.JBossTestCase;
import org.jboss.ejb3.test.service.ServiceOneRemote;
import org.jboss.ejb3.test.service.ServiceTwoRemote;
import org.jboss.ejb3.test.service.SessionRemote;
import org.jboss.jmx.adaptor.rmi.RMIAdaptor;

import javax.naming.InitialContext;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.Attribute;
import javax.management.AttributeList;
import java.util.Iterator;
import java.util.ArrayList;

/**
 * Sample client for the jboss container.
 *
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Id: ServiceUnitTestCase.java,v 1.6.2.4 2005/09/14 22:19:15 bill Exp $
 */

public class ServiceUnitTestCase
extends JBossTestCase
{
   org.apache.log4j.Category log = getLog();

   static boolean deployed = false;
   static int test = 0;

   public ServiceUnitTestCase(String name)
   {

      super(name);

   }

   public void testServiceWithDefaultRemoteJNDIName() throws Exception
   {
      ServiceOneRemote test = (ServiceOneRemote) getInitialContext().lookup(ServiceOneRemote.class.getName());
      test.setRemoteMethodCalls(0);
      final int count = 15;
      Thread[] threads = new Thread[count];
      for (int i = 0 ; i < count ; i++)
      {
         final int outer = i;
         threads[i] = new Thread(
               new Runnable()
               {
                  public void run()
                  {
                     try
                     {
                        ServiceOneRemote test = (ServiceOneRemote) getInitialContext().lookup(ServiceOneRemote.class.getName());
                        for (int j = 0 ; j < count ; j++)
                        {
                           String s = outer + "_" + j;
                           //System.out.println(s);
                           test.remoteMethod(s);
                        }
                     }
                     catch(Exception e)
                     {
                        throw new RuntimeException(e);
                     }
                  }
               }
            );
         threads[i].start();
      }

      Thread.sleep(5000);
      for (int i = 0 ; i < count ; i++)
      {
         threads[i].join();
      }
      assertEquals("There should only ever be one instance of the service", 1, test.getInstances());
      assertEquals("Wrong number of remote method calls", count * count, test.getRemoteMethodCalls());
   }

   public void testServiceWithDefaultLocalJNDIName() throws Exception
   {
	  MBeanServerConnection server = getServer();
      ObjectName testerName = new ObjectName("jboss.ejb3:service=Tester,test=service");
      Object[] params = {};
      String[] sig = {};
      server.invoke(testerName, "testServiceWithDefaultLocalJNDIName", params, sig);
   }

   public void testServiceWithRemoteBinding() throws Exception
   {
      ServiceTwoRemote test = (ServiceTwoRemote) getInitialContext().lookup("serviceTwo/remote");
      test.setCalled(false);
      assertFalse("Called should be false", test.getCalled());
      test.remoteMethod();
      assertTrue("Called should be true", test.getCalled());
   }

   public void testServiceWithLocalBinding() throws Exception
   {
	  MBeanServerConnection server = getServer();
      ObjectName testerName = new ObjectName("jboss.ejb3:service=Tester,test=service");
      Object[] params = {};
      String[] sig = {};
      server.invoke(testerName, "testServiceWithLocalBinding", params, sig);
   }

   public void testManagementServiceWithDefaultName() throws Exception
   {
	   MBeanServerConnection server = getServer();
      ObjectName testerName = new ObjectName("jboss.j2ee:service=EJB3,type=service,name=org.jboss.ejb3.test.service.ServiceOne");
      server.setAttribute(testerName, new Attribute("Attribute", new Integer(1234)));
      int ret = ((Integer)server.getAttribute(testerName, "Attribute"));
      assertEquals("wrong value for Attribute", 1234, ret);

      String s = (String)server.invoke(testerName, "jmxOperation", new Object[]{"1"}, new String[]{String.class.getName()});
      assertEquals("wrong return value", "x1x", s);

      String[] sa = (String[])server.invoke(testerName, "jmxOperation", new Object[]{new String[]{"0", "1"}}, new String[]{String[].class.getName()});
      assertEquals("wrong number of return values", 2, sa.length);
      for (int i = 0 ; i < sa.length ; i++)
      {
         assertEquals("Wrong return value index " + i, "x" + i + "x", sa[i]);
      }

      AttributeList attributes = new AttributeList();
      attributes.add(new Attribute("SomeAttr", new Integer(72)));
      attributes.add(new Attribute("OtherAttr", new Integer(99)));
      attributes = server.setAttributes(testerName, attributes);

      String[] attrs = new String[]{"Attribute", "SomeAttr", "OtherAttr"};
      AttributeList list = server.getAttributes(testerName, attrs);

      assertEquals("Wrong number of attributes returned", 3, list.size());
      for (Iterator it = list.iterator() ; it.hasNext() ; )
      {
         Attribute attr = (Attribute)it.next();
         String name = attr.getName();
         if (name.equals("Attribute"))
         {
            assertEquals("Wrong number for Attribute", 1234, attr.getValue());
         }
         else if (name.equals("SomeAttr"))
         {
            assertEquals("Wrong number for SomeAttr", 72, attr.getValue());
         }
         else if (name.equals("OtherAttr"))
         {
            assertEquals("Wrong number for OtherAttr", 99, attr.getValue());
         }
         else
         {
            throw new RuntimeException("Unknown attribute returned: " + name);
         }
      }

      server.setAttribute(testerName, new Attribute("WriteOnly", new Integer(2525)));
      assertEquals("Wrong read only value", 2525, ((Integer)server.getAttribute(testerName, "ReadOnly")).intValue());
   }

   public void testServiceDependencyInjectionAndInterception() throws Exception
   {
	   MBeanServerConnection server = getServer();
      ObjectName testerName = new ObjectName("jboss.ejb3:management=interface,with=customName,is=serviceThree");
      Boolean injected = (Boolean)server.getAttribute(testerName, "Injected");
      assertTrue("Injected should have been true", injected);

      Integer intercepted = (Integer)server.getAttribute(testerName, "Intercepted");;
      assertEquals("Wrong number of interceptions", 2, intercepted.intValue());
   }

   public void testCreationOrder() throws Exception
   {
	   MBeanServerConnection server = getServer();
      ObjectName testerName = new ObjectName("jboss.ejb3:service=Tester,test=service");
      ArrayList creates = (ArrayList)server.getAttribute(testerName, "Creates");
      ArrayList starts = (ArrayList)server.getAttribute(testerName, "Starts");

      //ServiceTwo does not implement any lifecycle so it won't appear
      int expectedLength = 4;
      assertEquals("Wrong length of created MBeans", expectedLength, creates.size());
      assertEquals("Wrong length of started MBeans", expectedLength, starts.size());

      for (int i = 0 ; i < expectedLength ; i++)
      {
         int create = Integer.parseInt((String)creates.get(i));
         int start = Integer.parseInt((String)starts.get(i));
         int expected = (i == 0) ? i + 1 : i + 2;
         assertEquals("Creation of Service" + create + "appears at the wronmg place", expected, create);
         assertEquals("Start of Service" + start + "appears at the wronmg place", expected, start);
      }
   }

   public void testDependsInjection() throws Exception
   {
	   MBeanServerConnection server = getServer();
      ObjectName testerName = new ObjectName("jboss.j2ee:service=EJB3,type=service,name=org.jboss.ejb3.test.service.ServiceFive");
      assertTrue("Proxy field was not injected", (Boolean)server.getAttribute(testerName, "InjectedProxyField"));
      assertTrue("Proxy method was not injected", (Boolean)server.getAttribute(testerName, "InjectedProxyMethod"));
      assertTrue("ObjectName field was not injected", (Boolean)server.getAttribute(testerName, "InjectedObjectNameField"));
      assertTrue("ObjectName method was not injected", (Boolean)server.getAttribute(testerName, "InjectedObjectNameMethod"));
      SessionRemote test = (SessionRemote) getInitialContext().lookup(SessionRemote.class.getName());
      assertTrue("StatelessBean @Depends was not injected", test.injectedDepends());
   }


   public static Test suite() throws Exception
   {
      return getDeploySetup(ServiceUnitTestCase.class, "service-test.sar, service-test.ejb3");
   }

}
