/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.stateful;

import javax.annotation.Resource;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.4.2.4 $
 */
@Stateless()
@TransactionManagement(TransactionManagementType.BEAN)
@Remote(Tester.class)
public class TesterBean implements Tester
{
   @Resource private UserTransaction ut;

   public void testSessionSynchronization() throws Exception
   {
      StatefulSyncLocal sfsb = (StatefulSyncLocal)new InitialContext().lookup(StatefulSyncLocal.class.getName());
      sfsb.setState("hello");
      sfsb.setBeforeCalled(false);

      ut.begin();
      sfsb.setState("world");
      ut.rollback();

      if (!sfsb.getState().equals("hello")) throw new RuntimeException("rollback didn't work");
      if (!sfsb.isBeforeCalled()) throw new RuntimeException("BEFORE WAS NOT CALLED");

      // test new proxy

      sfsb = (StatefulSyncLocal)new InitialContext().lookup(StatefulSyncLocal.class.getName());
      if (sfsb.getState() != null) throw new RuntimeException("State should be null: " + sfsb.getState());
   }
}
