/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.stateless.unit;

import javax.ejb.EJBException;
import javax.ejb.EJBAccessException;
import org.jboss.ejb3.test.stateless.CheckedStateless;
import org.jboss.ejb3.test.stateless.RunAsStateless;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SimplePrincipal;
import org.jboss.test.JBossTestCase;
import junit.framework.Test;

/**
 * @author <a href="mailto:bdecoste@jboss.com">William DeCoste</a>
 * @version $Revision: 1.3.2.4 $
 */
public class StatelessTestCase extends JBossTestCase
{
   org.apache.log4j.Category log = getLog();

   static boolean deployed = false;
   static int test = 0;

   public StatelessTestCase(String name)
   {
      super(name);
   }
 
   public void testCallerPrincipal() throws Exception
   {
      SecurityAssociation.setPrincipal(new SimplePrincipal("somebody"));
      SecurityAssociation.setCredential("password".toCharArray());
       
      RunAsStateless runAs = (RunAsStateless) getInitialContext().lookup("RunAsStatelessEjbName");
      assertNotNull(runAs);
      
      String principal = runAs.getCallerPrincipal();
      assertEquals("somebody", principal);
      
      principal = runAs.getCheckedCallerPrincipal();
      assertEquals("somebody", principal);
   }

   public void testRunAs() throws Exception
   {
      CheckedStateless checked = (CheckedStateless)getInitialContext().lookup(CheckedStateless.class.getName());
      
      SecurityAssociation.setPrincipal(new SimplePrincipal("somebody"));
      SecurityAssociation.setCredential("password".toCharArray());
      
      int result = checked.method(1);
      assertEquals(1,result);
      
      SecurityAssociation.setPrincipal(new SimplePrincipal("rolefail"));
      SecurityAssociation.setCredential("password".toCharArray());
      
      try {
         checked.method(2);
         assertTrue(false);
      } catch (Exception e){
         assertTrue(e instanceof EJBAccessException);
      }
      
      RunAsStateless runAs =
            (RunAsStateless) getInitialContext().lookup("RunAsStatelessEjbName");
      
      int i = runAs.method(3);
      assertEquals(3, i);
   } 

   public static Test suite() throws Exception
   {
      return getDeploySetup(StatelessTestCase.class, "stateless-test.ejb3");
   }
}