/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb3.test.txexceptions;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2.2.5 $
 */
@Stateless
@Remote(Dao.class)
public class DaoBean implements Dao
{
   @PersistenceContext EntityManager manager;

   public SimpleEntity get(int id)
   {
      SimpleEntity en = manager.getReference(SimpleEntity.class, id);
      en.getStuff(); // force ENF exception
      return en;
   }

   public void remove(int id)
   {
      manager.remove(manager.find(SimpleEntity.class, id));
   }

   public void createThrowAnnotatedAppException(int id) throws AnnotatedAppException
   {
      persist(id);
      throw new AnnotatedAppException();
   }

   private void persist(int id)
   {
      SimpleEntity entity = new SimpleEntity();
      entity.setId(id);
      entity.setStuff("stuff");
      manager.persist(entity);
   }

   public void createThrowAppException(int id) throws AppException
   {
      persist(id);
      throw new AppException();
   }

   public void createThrowCheckedRollbackException(int id) throws CheckedRollbackException
   {
      persist(id);
      throw new CheckedRollbackException();
   }

   public void createThrowNoRollbackRemoteException(int id) throws NoRollbackRemoteException
   {
      persist(id);
      throw new NoRollbackRemoteException();
   }

   public void createThrowNoRollbackRuntimeException(int id) throws NoRollbackRuntimeException
   {
      persist(id);
      throw new NoRollbackRuntimeException();
   }

   public void createThrowRollbackRemoteException(int id) throws RollbackRemoteException
   {
      persist(id);
      throw new RollbackRemoteException();
   }

   public void createThrowRollbackRuntimeException(int id) throws RollbackRuntimeException
   {
      persist(id);
      throw new RollbackRuntimeException();
   }
}
