/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.media.entity;

/**
 * Utility class for MediaEntityManager.
 */
public class MediaEntityManagerUtil
{
   /** Cached remote home (EJBHome). Uses lazy loading to obtain its value (loaded by getHome() methods). */
   private static org.jboss.media.entity.MediaEntityManagerHome cachedRemoteHome = null;

   /** Cached local home (EJBLocalHome). Uses lazy loading to obtain its value (loaded by getLocalHome() methods). */
   private static org.jboss.media.entity.MediaEntityManagerLocalHome cachedLocalHome = null;

   // Home interface lookup methods

   /**
    * Obtain remote home interface from default initial context
    * @return Home interface for MediaEntityManager. Lookup using JNDI_NAME
    */
   public static org.jboss.media.entity.MediaEntityManagerHome getHome() throws javax.naming.NamingException
   {
      if (cachedRemoteHome == null) {
         // Obtain initial context
         javax.naming.InitialContext initialContext = new javax.naming.InitialContext();
         try {
            java.lang.Object objRef = initialContext.lookup(org.jboss.media.entity.MediaEntityManagerHome.JNDI_NAME);
            cachedRemoteHome = (org.jboss.media.entity.MediaEntityManagerHome) javax.rmi.PortableRemoteObject.narrow(objRef, org.jboss.media.entity.MediaEntityManagerHome.class);
         } finally {
            initialContext.close();
         }
      }
      return cachedRemoteHome;
   }

   /**
    * Obtain remote home interface from parameterised initial context
    * @param environment Parameters to use for creating initial context
    * @return Home interface for MediaEntityManager. Lookup using JNDI_NAME
    */
   public static org.jboss.media.entity.MediaEntityManagerHome getHome( java.util.Hashtable environment ) throws javax.naming.NamingException
   {
      // Obtain initial context
      javax.naming.InitialContext initialContext = new javax.naming.InitialContext(environment);
      try {
         java.lang.Object objRef = initialContext.lookup(org.jboss.media.entity.MediaEntityManagerHome.JNDI_NAME);
         return (org.jboss.media.entity.MediaEntityManagerHome) javax.rmi.PortableRemoteObject.narrow(objRef, org.jboss.media.entity.MediaEntityManagerHome.class);
      } finally {
         initialContext.close();
      }
   }

   /**
    * Obtain local home interface from default initial context
    * @return Local home interface for MediaEntityManager. Lookup using JNDI_NAME
    */
   public static org.jboss.media.entity.MediaEntityManagerLocalHome getLocalHome() throws javax.naming.NamingException
   {
      // Local homes shouldn't be narrowed, as there is no RMI involved.
      if (cachedLocalHome == null) {
         // Obtain initial context
         javax.naming.InitialContext initialContext = new javax.naming.InitialContext();
         try {
            cachedLocalHome = (org.jboss.media.entity.MediaEntityManagerLocalHome) initialContext.lookup(org.jboss.media.entity.MediaEntityManagerLocalHome.JNDI_NAME);
         } finally {
            initialContext.close();
         }
      }
      return cachedLocalHome;
   }

}