/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mq.pm.jdbc2;

import javax.management.ObjectName;

import org.jboss.mq.pm.CacheStoreMBean;
import org.jboss.system.ServiceMBean;

/**
 * MBean interface.
 */
public interface PersistenceManagerMBean extends ServiceMBean, org.jboss.mq.pm.PersistenceManagerMBean, CacheStoreMBean
{
   /**
    * Get the instance of the persistence manager
    * 
    * @return the instance
    */
   Object getInstance();

   /**
    * Get the object name of the DataSource
    * 
    * @return the object name of the DataSource
    */
   ObjectName getConnectionManager();

   /**
    * Set the object name of the DataSource
    * 
    * @param connectionManagerName the object name of the DataSource
    */
   void setConnectionManager(ObjectName connectionManagerName);

   /**
    * Gets the sqlProperties.
    * 
    * @return Returns a Properties    
    */
   String getSqlProperties();

   /**
    * Sets the sqlProperties.
    * 
    * @param sqlProperties The sqlProperties to set    
    */
   void setSqlProperties(String value);

   /**
    * Sets the ConnectionRetryAttempts.
    * 
    * @param connectionRetryAttempts value    
    */
   void setConnectionRetryAttempts(int value);

   /**
    * Gets the ConnectionRetryAttempts.
    * 
    * @return Returns a ConnectionRetryAttempt value
    */
   int getConnectionRetryAttempts();

   /**
    * Any override recovery transaction timeout
    * 
    * @return the override transaction timeout
    */
   int getRecoveryTimeout();

   /**
    * Set the override recovery transaction timeout
    * 
    * @param timeout the timeout
    */
   void setRecoveryTimeout(int timeout);

   /**
    * Returns the recovery retries
    */
   public int getRecoveryRetries();

   /**
    * Set the recovery retries
    */
   public void setRecoveryRetries(int retries);
}
