/*  
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.ejb;

import java.util.Iterator;

import javax.management.ObjectName;

import org.jboss.deployment.SubDeployerExtMBean;
import org.jboss.mx.util.ObjectNameFactory;

/**
 * EJBDeployer MBean interface.
 */
public interface EJBDeployerMBean extends SubDeployerExtMBean
{
   /** The default ObjectName */
   static final ObjectName OBJECT_NAME = ObjectNameFactory.create("jboss.ejb:service=EJBDeployer");

   // Attributes ----------------------------------------------------
   
   /**
    * Whether ear deployments should be call by value
    */
   void setCallByValue(boolean callByValue);   
   boolean isCallByValue();

   /**
    * Enables/disables the application bean verification upon deployment.
    */
   void setVerifyDeployments(boolean verify);
   boolean getVerifyDeployments();
   
   /**
    * Enables/disables the verbose mode on the verifier.
    */
   void setVerifierVerbose(boolean verbose);
   boolean getVerifierVerbose();   

   /**
    * Enables/disables the strict mode on the verifier.
    */
   void setStrictVerifier(boolean strictVerifier);
   boolean getStrictVerifier();
   
   /**
    * Enables/disables the metrics interceptor for containers.
    */
   void setMetricsEnabled(boolean enable);
   boolean isMetricsEnabled();
   
   /**
    * Enables/disables the flag indicating that ejb-jar.dtd, jboss.dtd
    * and jboss-web.dtd conforming documents should be validated against the DTD.
    */
   void setValidateDTDs(boolean validate);
   boolean getValidateDTDs();
   
   /**
    * The dynamic class loading simple web server name.
    */
   void setWebServiceName(ObjectName webServiceName);
   ObjectName getWebServiceName();
   
   /**
    * The TransactionManagerServiceName.
    */
   void setTransactionManagerServiceName(ObjectName transactionManagerServiceName);
   ObjectName getTransactionManagerServiceName();

   // Operations ----------------------------------------------------
   
   /**
    * Returns the deployed applications.
    * 
    * @return an iterator over DeploymentInfo instances
    */
   Iterator listDeployedApplications();
   
}
