/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.deployment;

import java.net.URL;

import org.jboss.xb.binding.Unmarshaller;
import org.jboss.xb.binding.UnmarshallerFactory;
import org.jboss.xb.binding.sunday.unmarshalling.SchemaBinding;

/**
 * A simple subdeployer that deploys a managed object after parsing the 
 * deployment's xml file using an ObjectModelFactory.
 *
 * @author  <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.2.2.2 $
 */
public abstract class SchemaBindingSimpleSubDeployerSupport extends SimpleSubDeployerSupport
{
   /**
    * Get the schema binding 
    * 
    * @return the schema binding
    */
   public abstract SchemaBinding getSchemaBinding();

   protected void parseMetaData(DeploymentInfo di, URL url) throws DeploymentException
   {
      try
      {
         Unmarshaller unmarshaller = UnmarshallerFactory.newInstance().newUnmarshaller();
         di.metaData = unmarshaller.unmarshal(url.toString(), getSchemaBinding());
         if (di.metaData == null)
            throw new RuntimeException("The xml " + url + " is not well formed!");
      }
      catch (Throwable t)
      {
         DeploymentException.rethrowAsDeploymentException("Error parsing meta data " + url, t);
      }
   }
}
