/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.deployment.scanner;

/**
 * MBean interface.
 */
public interface DeploymentScannerMBean extends org.jboss.system.ServiceMBean
{

   /**
    * The ObjectName of the {@link Deployer} which we will use.
    * @param deployerName The object name of the deployer to use.
    */
   void setDeployer(javax.management.ObjectName deployerName);

   /**
    * Get the ObjectName of the {@link Deployer} which we are using.
    * @return The object name of the deployer we are using.
    */
   javax.management.ObjectName getDeployer();

   /**
    * Set the scan period for the scanner.
    * @param period This is the time in milliseconds between scans.
    * @throws IllegalArgumentException Period value out of range.
    */
   void setScanPeriod(long period);

   /**
    * Disable or enable the period based deployment scanning. <p>Manual scanning can still be performed by calling {@link #scan}.
    * @param flag True to enable or false to disable period based scanning.
    */
   void setScanEnabled(boolean flag);

   /**
    * Check if period based scanning is enabled.
    * @return True if enabled, false if disabled.
    */
   boolean isScanEnabled();

   /**
    * Scan for deployment changes.
    * @throws IllegalStateException Not initialized.
    * @throws Exception Scan failed.
    */
   void scan() throws java.lang.Exception;

}
