/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.system.server.jmx;

import javax.management.MBeanServerBuilder;
import javax.management.MBeanServerDelegate;
import javax.management.MBeanServer;

/**
 * An @link{MBeanServerBuilder} that creates a LazyMBeanServer instance to allow
 * the jboss MBeanServerImpl to be created when the ServerImpl is loaded and
 * the jboss jmx classes are available. It can be used at startup using
 * -Djavax.management.builder.initial=org.jboss.system.server.jmx.MBeanServerBuilderImpl
 * This is needed when enabling the jdk5 jconsole remote adaptor since this
 * adaptor initializes the platform MBeanServer before the jboss ServerImpl
 * is loaded.
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.1 $
 */
public class MBeanServerBuilderImpl
   extends MBeanServerBuilder
{
   /**
    * 
    * @return @{link MBeanServerDelegate}
    */ 
   public MBeanServerDelegate newMBeanServerDelegate()
   {
      return new MBeanServerDelegate();
   }

   /**
    * Creates a @{link LazyMBeanServer} as the MBeanServer. If the outer
    * parameter is null, the outer passed to the LazyMBeanServer is the
    * platform mbean server as obtained from
    * super.newMBeanServer(defaultDomain, outer, delegate).
    * 
    * @param defaultDomain
    * @param outer
    * @param delegate
    * @return LazyMBeanServer
    */ 
   public MBeanServer newMBeanServer(String defaultDomain, MBeanServer outer,
      MBeanServerDelegate delegate)
   {
      if( outer == null )
         outer = super.newMBeanServer(defaultDomain, outer, delegate);
      return new LazyMBeanServer(defaultDomain, outer, delegate);
   }
}
