/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.cts.test;

import java.rmi.RemoteException;

import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;

import org.jboss.test.JBossTestCase;
import org.jboss.test.cts.interfaces.StatelessSession;
import org.jboss.test.cts.interfaces.StatelessSessionHome;

/**
 *  Simple test to assert the correct exception when
 *  an instance cannot be created.<p>
 *  
 *  Put in a separate test because this currently
 *  stops the session bean from being usable at all.
 *
 *  @author adrian@jboss.com
 *  @version $Revision: 1.1.2.1 $
 */
public class StatelessSessionBrokenCreateUnitTestCase extends JBossTestCase
{
   public StatelessSessionBrokenCreateUnitTestCase(String name)
   {
      super(name);
   }

   public void testCreateExceptionFromRemoteInterface() throws Exception
   {
      getLog().debug("+++ testCreateExceptionFromInterface()");
      InitialContext ctx = new InitialContext();
      Object ref = ctx.lookup("ejbcts/StatelessSessionHome");
      StatelessSessionHome home = (StatelessSessionHome) PortableRemoteObject.narrow(ref, StatelessSessionHome.class);
      StatelessSession sessionBean = home.create();
      sessionBean.breakCreate();
      ObjectName pool = new ObjectName("jboss.j2ee:jndiName=ejbcts/StatelessSessionHome,plugin=pool,service=EJB");
      getServer().invoke(pool, "clear", null, null);
      try
      {
         sessionBean.method1("This should cause a CreateException");
         fail("Should not be here");
      }
      catch (RemoteException expected)
      {
         log.debug("Expected", expected);
      }
   }

   public static Test suite() throws Exception
   {
      return getDeploySetup(StatelessSessionBrokenCreateUnitTestCase.class, "cts.jar");
   }
}
