/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.test.jca.ejb;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import org.jboss.logging.Logger;
import org.jboss.test.jca.adapter.TestConnection;
import org.jboss.test.jca.adapter.TestConnectionFactory;

/**
 * UserTxSessionBean.java
 *
 * @author <a href="mailto:d_jencks@users.sourceforge.net">David Jencks</a>
 * @version <tt>$Revision: 1.1.18.2 $</tt>
 *
 * @ejb.bean   name="UserTxSession"
 *             jndi-name="UserTxSession"
 *             local-jndi-name="LocalUserTxSession"
 *             view-type="both"
 *             type="Stateless"
 *             transaction-type="Bean"
 * @ejb.transaction type="NotSupported"
 */
public class UserTxSessionBean 
   implements SessionBean  
{

   /** The serialVersionUID */
   private static final long serialVersionUID = 1L;
   private SessionContext ctx;
   private Logger log = Logger.getLogger(getClass());

   public UserTxSessionBean() 
   {
      
   }
   
   /**
    * Describe <code>testUserTxJndi</code> method here.
    *
    * @return a <code>boolean</code> value
    *
    * @ejb:interface-method
    */
   public boolean testUserTxJndi()
   {
      try 
      {
         TestConnectionFactory tcf = (TestConnectionFactory)new InitialContext().lookup("java:/JBossTestCF");
         TestConnection tc = (TestConnection)tcf.getConnection();
         UserTransaction ut = (UserTransaction)new InitialContext().lookup("UserTransaction");
         ut.begin();
         boolean result = tc.isInTx();
         log.info("Jndi test, inTx: " + result);
         ut.commit();
         tc.close();
         return result;
      }
      catch (Exception e)
      {
         throw new EJBException(e.getMessage());
      }
      
   }

   /**
    * Describe <code>testUserTxSessionCtx</code> method here.
    *
    * @return a <code>boolean</code> value
    *
    * @ejb:interface-method
    */
   public boolean testUserTxSessionCtx()
   {
      try 
      {
         TestConnectionFactory tcf = (TestConnectionFactory)new InitialContext().lookup("java:/JBossTestCF");
         TestConnection tc = (TestConnection)tcf.getConnection();
         UserTransaction ut = ctx.getUserTransaction();
         ut.begin();
         boolean result = tc.isInTx();
         log.info("ctx test, inTx: " + result);
         ut.commit();
         tc.close();
         return result;
      }
      catch (Exception e)
      {
         throw new EJBException(e.getMessage());
      }
      
   }
   
   /**
    * @ejb:interface-method
    */
   public void testUnclosedError() throws Exception
   {
      TestConnectionFactory tcf = (TestConnectionFactory)new InitialContext().lookup("java:/JBossTestCF");
      tcf.getConnection(); // DONT CLOSE 
   }

   public void ejbCreate() 
   {
   }

   public void ejbActivate()
   {
    }

   public void ejbPassivate()
   {
   }

   public void ejbRemove()
   {
   }

   public void setSessionContext(SessionContext ctx)
   {
      this.ctx = ctx;
   }

   public void unsetSessionContext()
   {
      this.ctx = null;
   }

}
