/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.jca.test;

import javax.transaction.Status;
import javax.transaction.Transaction;

/**
 * Abstract contentious pooling stress test.
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.1.2.3 $
 */
public class AbstractContentiousRecyclingPoolingStressTest extends AbstractPoolingStressTest
{
   TransactionState[] txStates;
   
   public void runConcurrentTest(ConcurrentRunnable[] runnables, ConcurrentTestCallback callback) throws Throwable
   {
      // HACK: The default config won't tweak this test
      int transactions = getBeanCount();
      int threads = getThreadCount();
      if (transactions >= threads)
         transactions = threads / 2;
      if (transactions == 0)
         transactions = 1;
      
      txStates = new TransactionState[transactions];
      for (int i = 0; i < txStates.length; ++i)
         txStates[i] = new TransactionState();
      super.runConcurrentTest(runnables, callback);
   }

   public abstract class ContentiousRecyclingPoolingRunnable extends ConcurrentRunnable
   {
      TransactionState txState;
      
      public ContentiousRecyclingPoolingRunnable()
      {
      }

      public void doStart() throws Throwable
      {
         txState = txStates[id % txStates.length];
      }

      public void doEnd() throws Throwable
      {
      }

      public void doRun() throws Throwable
      {
         waitDone();
         txState.begin();
         try
         {
            doRun1();
         }
         catch (Throwable t)
         {
            setFailure(t);
            tm.setRollbackOnly();
            throw t;
         }
         finally
         {
            tm.suspend();
            waitDone();
            txState.commit();
         }
      }
      
      public abstract void doRun1() throws Throwable;
   }

   public class TransactionState
   {
      Transaction tx;

      public TransactionState()
      {
      }
      
      public synchronized void begin() throws Throwable
      {
         if (tx == null)
         {
            tm.begin();
            tx = tm.getTransaction();
         }
         else
         {
            tm.resume(tx);
         }
      }
      
      public synchronized void commit() throws Throwable
      {
         if (tx != null)
         {
            tm.resume(tx);
            if (tm.getStatus() == Status.STATUS_MARKED_ROLLBACK)
               tm.rollback();
            else
               tm.commit();
            tx = null;
         }
      }
   }
   
   public AbstractContentiousRecyclingPoolingStressTest(String name)
   {
      super(name);
   }
}
