/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 *
 */
package org.jboss.test.jca.test;

import java.rmi.RemoteException;

import javax.management.Attribute;
import javax.management.ObjectName;
import javax.resource.ResourceException;

import junit.framework.Test;

import org.jboss.test.JBossTestCase;
import org.jboss.test.jca.interfaces.UserTxSession;
import org.jboss.test.jca.interfaces.UserTxSessionHome;

/**
 * Abstract concurrent stress test.
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.1.2.1 $
 */
public class CachedConnectionErrorUnitTestCase extends JBossTestCase
{
   public CachedConnectionErrorUnitTestCase (String name)
   {
      super(name);
   }

   protected void setError(Boolean value) throws Exception
   {
      ObjectName CCM = new ObjectName("jboss.jca:service=CachedConnectionManager");
      getServer().setAttribute(CCM, new Attribute("Error", value));
   }

   protected void setUp() throws Exception
   {
      setError(Boolean.TRUE);
   }

   protected void tearDown() throws Exception
   {
      setError(Boolean.FALSE);
   }

   public static Test suite() throws Exception
   {
      Test t1 = getDeploySetup(CachedConnectionErrorUnitTestCase.class, "jcatest.jar");
      Test t2 = getDeploySetup(t1, "testadapter-ds.xml");
      return getDeploySetup(t2, "jbosstestadapter.rar");
   }

   public void testCachedConnectionError() throws Exception
   {
      UserTxSessionHome sh = (UserTxSessionHome) getInitialContext().lookup("UserTxSession");
      UserTxSession s = sh.create();
      try
      {
         s.testUnclosedError();
         fail("Should not be here");
      }
      catch (RemoteException expected)
      {
         log.debug("Got the expected exception RemoteException:", expected);
         Throwable linked = expected.getCause();
         assertTrue("Not the expected exception: " + linked, linked != null && linked instanceof ResourceException);
      }
   }
}
