/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.jmx.test;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.apache.log4j.Logger;
import org.jboss.test.JBossTestCase;

/**
 * Tests for XMLAttributePersistenceManager
 * 
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.2.2 $
 */
public class XMLAttributePersistenceManagerUnitTestCase extends JBossTestCase
{
   public XMLAttributePersistenceManagerUnitTestCase(String name)
   {
      super(name);
   }

   /**
    * [JBAS-1988], test we can write/read to a directory that contains spaces in its name. 
    * 
    * @see org.jboss.test.jmx.xmbean.XMLAttributePersistenceManagerTestService
    */
   public void testWriteToPathNameContainingSpaces() throws Exception
   {
      Logger log = getLog();
      log.info("+++ testWriteToPathNameContainingSpaces");

      String testService = "xmlapm-xmbean.sar";
      
      try
      {
         deploy(testService);
         
         MBeanServerConnection server = super.getServer();
         ObjectName target = new ObjectName("jboss.test:service=XMLAttributePersistenceManagerTestService");
         
         // Store some attributes under an id
         AttributeList atlist = new AttributeList();
         String storeId = "testId";
         Integer anInteger = new Integer(666);
         String aString = new String("Evil Test");
         atlist.add(new Attribute("Attr1", anInteger));
         atlist.add(new Attribute("Attr2", aString));
       
         getLog().info("Storing AttributeList");
         server.invoke(
               target,
               "store",
               new Object[] { storeId, atlist },
               new String[] { storeId.getClass().getName(), atlist.getClass().getName() }
               );
         
         getLog().info("Loading AttributeList");
         server.invoke(
               target,
               "load",
               new Object[] { storeId },
               new String[] { storeId.getClass().getName() }
               );         
         
      }
      catch (Exception e)
      {
         getLog().warn("Caught exception", e);
         fail("Unexcepted Exception, see the Log file");
      }
      finally
      {
         undeploy(testService);
      }
   }     
}