/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.kernel.deployment.jboss.beans.servicepojo;

/**
 * Supposedly a base class for services
 *
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.2.1 $
 */
public abstract class AbstractService
{
   public static final String CONSTRUCTED = "CONSTRUCTED";
   public static final String CREATED     = "CREATED";
   public static final String STARTED     = "STARTED";
   public static final String STOPPED     = "STOPPED";
   public static final String DESTROYED   = "DESTROYED";
   public static final String FAILED      = "FAILED";
   
   // Protected -----------------------------------------------------
   
   protected String name;
   protected String state = CONSTRUCTED;
   
   // Constructor ---------------------------------------------------
   
   public AbstractService(String name)
   {
      this.name = name;
      log("CTOR");
   }
   
   // Accessors -----------------------------------------------------

   public String getName()
   {
      return name;
   }
   
   public String getState()
   {
      return state;
   }
   
   // Lifecycle -----------------------------------------------------
   
   public void create() throws Exception
   {
      state = CREATED;
      log("create()");
   }
   
   public void start() throws Exception
   {
      state = STARTED;
      log("start()");
   }
   
   public void stop() throws Exception
   {
      state = STOPPED;
      log("stop()");
   }
   
   public void destroy() throws Exception
   {
      state = DESTROYED;
      log("destroy()");
   }
   
   // Just to avoid org.jboss.logging -------------------------------
   
   public void log(Object message)
   {
      System.out.println(getName() + " - " + message);
   }
   
   // Overrides -----------------------------------------------------
   
   public String toString()
   {
      StringBuffer sbuf = new StringBuffer();
      sbuf
      .append(getClass().getName())
      .append("[ name=").append(name)
      .append(", state=").append(state)
      .append(" ]");
      
      return sbuf.toString();
   }
}
