/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.kernel.deployment.jboss.beans.servicepojo;

import java.util.List;

/**
 * A simple pojo to represent a hypothetical service
 *
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.2.1 $
 */
public class POJOService extends AbstractService
{
   // Private -------------------------------------------------------
   
   private PoolService pool;
   private List addresses;
   
   // Constructor ---------------------------------------------------
   
   public POJOService()
   {
      super("POJOService");
   }
   
   // Accessors/Mutators --------------------------------------------

   public void setPoolService(PoolService pool)
   {
      log("setPoolService(" + pool + ")");      
      this.pool = pool;
   }

   public void setBindAddresses(List addresses)
   {
      log("setBindAddresses(" + addresses + ")");       
      this.addresses = addresses;
   }
   
   // Overrides -----------------------------------------------------
   
   public void create() throws Exception
   {
      super.create();
      
      if (!(pool.getState().equals(CREATED)))
      {
         state = FAILED;
         throw new IllegalStateException("pool not CREATED");
      }
   }
   
   public void start() throws Exception
   {
      super.start();
      
      if (!(pool.getState().equals(STARTED)))
      {
         state = FAILED;
         throw new IllegalStateException("pool not STARTED");
      }
   }
   
   public void stop() throws Exception
   {
      super.stop();
      
      if (!(pool.getState().equals(STARTED)))
      {
         state = FAILED;
         throw new IllegalStateException("pool not STARTED");
      }
   }
   
   public void destroy() throws Exception
   {
      super.destroy();
      
      if (!(pool.getState().equals(STOPPED)))
      {
         state = FAILED;
         throw new IllegalStateException("pool not STOPPED");
      }
   }
   
   public String toString()
   {
      StringBuffer sbuf = new StringBuffer();
      sbuf
      .append(getClass().getName())
      .append("[ name=").append(name)
      .append(", state=").append(state)      
      .append(", pool=").append(pool)
      .append(", addresses=").append(addresses)
      .append(" ]");
      
      return sbuf.toString();
   }
}
