/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.kernel.deployment.jboss.test;

import java.util.List;
import java.util.Properties;

import javax.naming.InitialContext;

import org.jboss.test.JBossTestCase;
import org.jboss.test.kernel.deployment.jboss.beans.configpojo.ConfigPOJO;

/**
 * Deployment tests.
 * 
 * @author <a href="dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.2.1 $
 */
public class ConfigPOJOUnitTestCase extends JBossTestCase
{
   public ConfigPOJOUnitTestCase(String test)
   {
      super(test);
   }
   
   public void testConfigPOJO() throws Exception
   {
      try
      {
         deploy("testkernel-configpojo.beans");
         
         InitialContext ctx = new InitialContext();
         ConfigPOJO config = (ConfigPOJO) ctx.lookup("test/kernel/deployment/config/pojo");
         assertEquals("joe", config.getUserId());
         assertEquals("secret", config.getPassword());
         List roles = config.getRoles();
         assertEquals("trader", (String)roles.get(0));
         assertEquals("manager", (String)roles.get(1));
      }
      catch (Exception e)
      {
         getLog().info("Caught exception", e);
         throw e;
      }
      finally
      {
         undeploy("testkernel-configpojo.beans");
      }
   }
   
   public void testConfigProperties() throws Exception
   {
      try
      {
         deploy("testkernel-configproperties.beans");
         
         InitialContext ctx = new InitialContext();
         Properties props = (Properties) ctx.lookup("test/kernel/deployment/config/properties");
         assertEquals("value1", props.getProperty("config.property1"));
         assertEquals("value2", props.getProperty("config.property2"));
         assertEquals("value3", props.getProperty("config.property3"));
      }
      catch (Exception e)
      {
         getLog().info("Caught exception", e);
         throw e;
      }      
      finally
      {
         undeploy("testkernel-configproperties.beans");
      }
   }   
}
