/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.test.security.test;

import java.io.FilePermission;
import java.net.URL;
import java.security.CodeSource;
import java.security.Permission;
import java.security.PermissionCollection;
import java.security.Policy;
import java.security.Principal;

import javax.security.auth.Subject;

import junit.framework.TestCase;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SimplePrincipal;

/**
 * Test that the security context thread locals do NOT propagate to child threads
 *
 @author Scott.Stark@jboss.org
 @version $Revision: 1.1 $
 */
public class SAThreadLocalUnitTestCase extends TestCase
{
   private Principal authPrincipal;
   private Subject authSubject;

   public SAThreadLocalUnitTestCase(String name)
   {
      super(name);
   }

   /**
    * Test the order of PermissionNames
    */
   public void testSecurityContext()
   {
      authPrincipal = new SimplePrincipal("jduke");
      authSubject = new Subject();
      authSubject.getPrincipals().add(authPrincipal);
      SecurityAssociation.pushSubjectContext(authSubject, authPrincipal, "theduke");
      validateSettings(false);
   }
   public void testThreadLocal() throws Exception
   {
      testSecurityContext();
      TestThread t = new TestThread("testThreadLocal", true);
      t.start();
      t.join();
      if( t.error != null )
      {
         t.error.printStackTrace();
         fail("TestThread saw an error");
      }
   }

   /**
    * SecurityAssociation.getSubject() == authSubject
    * SecurityAssociation.getPrincipal() == authPrincipal
    */
   private void validateSettings(boolean expectNull)
   {
      Subject s = SecurityAssociation.getSubject();
      Principal p = SecurityAssociation.getPrincipal();

      if( expectNull )
      {
         assertNull("getSubject() == null", s);
         assertNull("getPrincipal() == null", p);         
      }
      else
      {
         assertTrue("getSubject() == authSubject", authSubject.equals(s));
         assertTrue("getPrincipal() == authPrincipal", authPrincipal.equals(p));
      }
   }
   
   class TestThread extends Thread
   {
      Throwable error;
      boolean expectNull;
      TestThread(String name, boolean expectNull)
      {
         super(name);
         this.expectNull = expectNull;
      }

      public void run()
      {
         try
         {
            validateSettings(expectNull);
         }
         catch(Throwable e)
         {
            error = e;
         }
      }
   }

   protected void setUp()
   {
      System.setProperty("org.jboss.security.SecurityAssociation.ThreadLocal", "true");
      SecurityAssociation.setServer();
   }
}
