/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.test.testbyvalue.test;

import java.util.Date;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;

import org.jboss.test.JBossTestCase;
import org.jboss.test.testbyvalue.interfaces.RootStatelessSessionHome;

/**
* @author Clebert Suconic
*/
public class ByValueUnitTestCase
extends JBossTestCase
{
   static boolean deployed = false;
   static int test = 0;
   static Date startDate = new Date();

   protected final String namingFactory =
   System.getProperty(Context.INITIAL_CONTEXT_FACTORY);

   protected final String providerURL =
   System.getProperty(Context.PROVIDER_URL);
    private static final int ITERATIONS = 1000;

    public ByValueUnitTestCase(String name) {
      super(name);
   }

    public void testByValue() throws Exception
    {
        InitialContext ctx = new InitialContext();
        Object obj = ctx.lookup("RootTestByValue");

        RootStatelessSessionHome home = (RootStatelessSessionHome) PortableRemoteObject.narrow(obj, RootStatelessSessionHome.class);
        long time = home.create().doTestByValue(ITERATIONS);
        // I really wanted System.out as I wanted this to be sent to the XML output for ant junit tests
        System.out.println("Time spent into CallByValues over "+ITERATIONS+" iterations=" + time);
    }

    public void testByValueOnEntity() throws Exception
    {
        InitialContext ctx = new InitialContext();
        Object obj = ctx.lookup("RootTestByValue");

        RootStatelessSessionHome home = (RootStatelessSessionHome) PortableRemoteObject.narrow(obj, RootStatelessSessionHome.class);
        long time = home.create().doTestEntity(ITERATIONS);
        // I really wanted System.out as I wanted this to be sent to the XML output for ant junit tests
        System.out.println("Time spent into CallByValues over "+ITERATIONS+" iterations=" + time);
    }

    public void testByReferenceOnEntity() throws Exception
    {
        InitialContext ctx = new InitialContext();
        Object obj = ctx.lookup("RootTestByValue");

        RootStatelessSessionHome home = (RootStatelessSessionHome) PortableRemoteObject.narrow(obj, RootStatelessSessionHome.class);
        long time = home.create().doTestEntityByReference(ITERATIONS);
        // I really wanted System.out as I wanted this to be sent to the XML output for ant junit tests
        System.out.println("Time spent into CallByValues over "+ITERATIONS+" iterations=" + time);
    }

    public void testByReference() throws Exception
    {
        InitialContext ctx = new InitialContext();
        Object obj = ctx.lookup("RootTestByValue");

        RootStatelessSessionHome home = (RootStatelessSessionHome) PortableRemoteObject.narrow(obj, RootStatelessSessionHome.class);
        long time = home.create().doTestByReference(ITERATIONS);
        // I really wanted System.out as I wanted this to be sent to the XML output for ant junit tests
        System.out.println("Time spent into CallByReferences over "+ITERATIONS+" iterations=" + time);
    }

   public static Test suite() throws Exception
   {
      Test t1 = getDeploySetup(ByValueUnitTestCase.class, "jar-byvalue.jar");
      return t1;
   }

}
