/*
* JBoss, Home of Professional Open Source
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.test.webservice;

import org.jboss.logging.Logger;
import org.jboss.test.JBossTestCase;
import org.jboss.test.webservice.samples.OrganizationEJBEndpoint;

import javax.management.ObjectName;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;

import java.io.IOException;
import java.util.Properties;

import junit.framework.Test;
import junit.framework.TestSuite;

/** Common functionality for web services test cases.
 *
 * @author Thomas.Diesler@jboss.org
 * @version $Revision: 1.1.2.4 $
 */
public class WebserviceTestBase extends JBossTestCase
{
  
   public WebserviceTestBase(String name)
   {
      super(name);
   }

   /**
    * Get the client's env context, see tracker [840598] for details
    */
   protected InitialContext getClientContext() throws NamingException
   {
      Properties env = new Properties();
      env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
      env.setProperty(Context.URL_PKG_PREFIXES, "org.jboss.naming.client");
      env.setProperty(Context.PROVIDER_URL, "jnp://" + getServerHost() + ":1099");
      env.setProperty("j2ee.clientName", "ws4ee-client");
      return new InitialContext(env);
   }

   /** Return true if the new JBossWS stack is available
    */
   public static boolean isJBossWSAvailable()
   {
      try
      {
         ServiceFactory factory = ServiceFactory.newInstance();
         if ("org.jboss.ws.jaxrpc.ServiceFactoryImpl".equals(factory.getClass().getName()))
            return true;
      }
      catch (ServiceException e)
      {
         // ignore
      }
      return false;
   }

   /** Return true if the Axis based stack is available
    */
   public static boolean isWS4EEAvailable()
   {
      try
      {
         ServiceFactory factory = ServiceFactory.newInstance();
         if ("org.jboss.webservice.client.ServiceFactoryImpl".equals(factory.getClass().getName()))
            return true;
      }
      catch (ServiceException e)
      {
         // ignore
      }
      return false;
   }

   public static Test getDeploySetupForJBossWS(final Class clazz, String jarName) throws Exception
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new TestSuite(clazz));
      if (isJBossWSAvailable() == false)
      {
         jarName = null;
      }
      return getDeploySetup(suite, jarName);
   }
   
   public static Test getDeploySetupForWS4EE(final Class clazz, String jarName) throws Exception
   {
      TestSuite suite = new TestSuite();
      suite.addTest(new TestSuite(clazz));
      if (isWS4EEAvailable() == false)
      {
         jarName = null;
      }
      return getDeploySetup(suite, jarName);
   }
}
