/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.webservice.addressrewrite;

// $Id: AddressRewriteTestCase.java,v 1.1.2.4 2005/06/28 09:53:48 tdiesler Exp $

import java.net.URL;

import javax.management.Attribute;
import javax.management.ObjectName;
import javax.xml.namespace.QName;
import javax.xml.rpc.Call;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceFactory;

import org.jboss.test.JBossTestCase;

/**
 * Test soap:address rewriting in the WSDL.
 *
 * @author jason.greene@jboss.com
 */
public class AddressRewriteTestCase extends JBossTestCase
{
   private static String NAMESPACE = "http://test.jboss.org/ws4eesimple";
   
   private String WSDL_LOCATION = "http://" + getServerHost() + ":8080/ws4ee-addressrewrite?wsdl";

   private ObjectName objectName;
   private Boolean alwaysModifySOAPAddress;
   private String wsHost;

   /**
    * Construct the test case with a given name
    */
   public AddressRewriteTestCase(String name)
   {
      super(name);
   }

   public void setUp() throws Exception
   {
      ObjectName ws4eeServiceName = new ObjectName("jboss.ws4ee:service=AxisService");
      ObjectName jbosswsServiceName = new ObjectName("jboss.ws:service=ServiceEndpointManager");

      if (getServer().isRegistered(ws4eeServiceName))
         objectName = ws4eeServiceName;
      else if (getServer().isRegistered(jbosswsServiceName))
         objectName = jbosswsServiceName;

      alwaysModifySOAPAddress = (Boolean)getServer().getAttribute(objectName, "AlwaysModifySOAPAddress");
      wsHost = (String)getServer().getAttribute(objectName, "WebServiceHost");
   }

   public void tearDown() throws Exception
   {
      delegate.undeploy("ws4ee-addressrewrite.war");
      Attribute attr = new Attribute("AlwaysModifySOAPAddress", alwaysModifySOAPAddress);
      getServer().setAttribute(objectName, attr);
   }

   public void setupAlwaysModify(Boolean value) throws Exception
   {
      Attribute attr = new Attribute("AlwaysModifySOAPAddress", value);
      getServer().setAttribute(objectName, attr);
      delegate.redeploy("ws4ee-addressrewrite.war");
   }

   public void testRewrite() throws Exception
   {
      setupAlwaysModify(new Boolean(true));
      ServiceFactory serviceFactory = ServiceFactory.newInstance();

      Service service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "ValidURLService"));
      Call call = (Call)service.createCall(new QName(NAMESPACE, "ValidURLPort"), "sayHello");
      assertEquals("http://" + wsHost + ":8080/ws4ee-addressrewrite", call.getTargetEndpointAddress());

      service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "InvalidURLService"));
      call = (Call)service.createCall(new QName(NAMESPACE, "InvalidURLPort"), "sayHello");
      assertEquals("http://" + wsHost + ":8080/ws4ee-addressrewrite", call.getTargetEndpointAddress());

      service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "ValidSecureURLService"));
      call = (Call)service.createCall(new QName(NAMESPACE, "ValidSecureURLPort"), "sayHello");
      assertEquals("https://" + wsHost + ":8443/ws4ee-addressrewrite", call.getTargetEndpointAddress());

      service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "InvalidSecureURLService"));
      call = (Call)service.createCall(new QName(NAMESPACE, "InvalidSecureURLPort"), "sayHello");
      assertEquals("https://" + wsHost + ":8443/ws4ee-addressrewrite", call.getTargetEndpointAddress());
   }

   public void testNoRewrite() throws Exception
   {
      setupAlwaysModify(new Boolean(false));
      ServiceFactory serviceFactory = ServiceFactory.newInstance();

      Service service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "ValidURLService"));
      Call call = (Call)service.createCall(new QName(NAMESPACE, "ValidURLPort"), "sayHello");
      assertEquals("http://somehost:80/somepath", call.getTargetEndpointAddress());

      service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "InvalidURLService"));
      call = (Call)service.createCall(new QName(NAMESPACE, "InvalidURLPort"), "sayHello");
      assertEquals("http://" + wsHost + ":8080/ws4ee-addressrewrite", call.getTargetEndpointAddress());

      service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "ValidSecureURLService"));
      call = (Call)service.createCall(new QName(NAMESPACE, "ValidSecureURLPort"), "sayHello");
      assertEquals("https://somehost:443/some-secure-path", call.getTargetEndpointAddress());

      service = serviceFactory.createService(new URL(WSDL_LOCATION), new QName(NAMESPACE, "InvalidSecureURLService"));
      call = (Call)service.createCall(new QName(NAMESPACE, "InvalidSecureURLPort"), "sayHello");
      assertEquals("https://" + wsHost + ":8443/ws4ee-addressrewrite", call.getTargetEndpointAddress());
   }
}