/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.webservice.secure.noenv;

// $Id: TransportOptionsTestCase.java,v 1.1.4.4 2005/10/23 12:15:43 starksm Exp $

import java.io.File;
import java.lang.reflect.UndeclaredThrowableException;
import java.net.SocketException;

import javax.naming.Context;
import javax.xml.rpc.Service;
import javax.xml.rpc.Stub;

import junit.framework.Test;

import org.jboss.test.webservice.WebserviceTestBase;
import org.jboss.test.webservice.ws4eesimple.HelloWs;

/**
 * Tests SSL transport options 
 *
 * @author <a href="mailto:jason.greene@jboss.com">Jason T. Greene</a>
 */
public class TransportOptionsTestCase extends WebserviceTestBase
{
 
   /**
    * Construct the test case with a given name
    */
   public TransportOptionsTestCase(String name)
   {
      super(name);
   }

   /**
    * deploy the test archives
    */
   public static Test suite() throws Exception
   {
      return getDeploySetup(TransportOptionsTestCase.class, "ws4ee-simple-secure.war, ws4ee-simple-secure-client.jar");
   }

   /**
    * Test client application access
    */
   public void testWithTransportOptions() throws Exception
   {
      Context envCtx = getClientContext();
      Service service = (Service)envCtx.lookup("java:comp/env/service/HelloWsSecureService");
      HelloWs port = (HelloWs)service.getPort(HelloWs.class);

      String keyStore = "resources/test-configs/webservice-ssl/conf/client.keystore";
      assertTrue("Cannot find keystore", new File(keyStore).exists());

      Stub stub = (Stub)port;
      stub._setProperty("org.jboss.webservice.keyStore", keyStore);
      stub._setProperty("org.jboss.webservice.keyStorePassword", "unit-tests-client");
      stub._setProperty("org.jboss.webservice.keyStoreType", "JKS");
      stub._setProperty("org.jboss.webservice.trustStore", keyStore);
      stub._setProperty("org.jboss.webservice.trustStorePassword", "unit-tests-client");
      stub._setProperty("org.jboss.webservice.trustStoreType", "JKS");

      String res = port.sayHello("Hello");
      assertEquals("'Hello' to you too!", res);

      res = port.sayHello("Hello2");
      assertEquals("'Hello2' to you too!", res);
   }

   /**
    * Test client application access
    */
   public void testWithoutKeystore() throws Exception
   {
      Context envCtx = getClientContext();
      Service service = (Service)envCtx.lookup("java:comp/env/service/HelloWsSecureService");
      HelloWs port = (HelloWs)service.getPort(HelloWs.class);

      String keyStore = "resources/test-configs/webservice-ssl/conf/client.keystore";
      File storeFile = new File(keyStore);
      log.info("Using trustStore: "+storeFile.getAbsolutePath());
      assertTrue("Cannot find keystore", storeFile.exists());

      Stub stub = (Stub)port;
      stub._setProperty("org.jboss.webservice.trustStore", keyStore);
      stub._setProperty("org.jboss.webservice.trustStorePassword", "unit-tests-client");
      stub._setProperty("org.jboss.webservice.trustStoreType", "JKS");

      boolean exception = false;
      try
      {
         String res = port.sayHello("Hello");
         assertEquals("'Hello' to you too!", res);
      }
      catch (UndeclaredThrowableException e)
      {
         Throwable t = e.getUndeclaredThrowable();
         log.info("Saw UndeclaredThrowableException, t="+t.getClass());
         if (t instanceof SocketException)
            exception = true;
      }

      assertTrue("SocketException expected", exception);
   }
}
