/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.xslt.support;

import java.io.StringReader;
import java.lang.reflect.Method;
import java.lang.reflect.UndeclaredThrowableException;
import java.util.Hashtable;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;

import org.jboss.system.ServiceMBeanSupport;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

/**
 * A test mbean service.
 * 
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.2.3 $
 */
public class XalanCheck extends ServiceMBeanSupport
   implements XalanCheckMBean
{
   // Constructors --------------------------------------------------
  
   /**
    * CTOR
   **/
   public XalanCheck()
   {
      // empty
   }

   // Attributes -----------------------------------------------------
   
   public String getXalanVersion()
   {
      try
      {
         ClassLoader loader = Thread.currentThread().getContextClassLoader();
         Class c = loader.loadClass("org.apache.xalan.Version");
         Object v = c.newInstance();
         Class[] sig = {};
         Method getVersion = c.getDeclaredMethod("getVersion", sig);
         Object[] args = {};
         String version = (String) getVersion.invoke(v, args);
         return version;
      }
      catch(Throwable e)
      {
         throw new UndeclaredThrowableException(e);
      }      
   }
   
   // Operations -----------------------------------------------------
  
   public Hashtable fetchXalanEnvironmentHash()
   {
      try
      {
         ClassLoader loader = Thread.currentThread().getContextClassLoader();
         Class c = loader.loadClass("org.apache.xalan.xslt.EnvironmentCheck");
         Object envc = c.newInstance();
         Class[] sig = {};
         Method getEnvironmentHash = c.getDeclaredMethod("getEnvironmentHash", sig);
         Object[] args = {};
         Hashtable htab = (Hashtable) getEnvironmentHash.invoke(envc, args);
         return htab;
      }
      catch(Throwable e)
      {
         throw new UndeclaredThrowableException(e);
      }
   }
   
   /**
    * Throws an exception when run using xalan 2.5.2
    * Borrowed from here: http://issues.apache.org/bugzilla/show_bug.cgi?id=15140
    */
   public void testXalan25Bug15140() throws Exception
   {
       String testString = "<doc xmlns:a=\"http://www.test.com\"/>";
       
       SAXParserFactory parserFactory
         = SAXParserFactory.newInstance();
       SAXParser parser = parserFactory.newSAXParser();
       XMLReader reader = parser.getXMLReader();
       reader.setFeature("http://xml.org/sax/features/namespaces", true);
       reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true); 

       DOMResult domResult = new DOMResult();
       SAXTransformerFactory transformerFactory
         = (SAXTransformerFactory) TransformerFactory.newInstance();
       TransformerHandler handler = transformerFactory.newTransformerHandler();
       handler.setResult(domResult);

       reader.setContentHandler(handler);

       InputSource input = new InputSource(new StringReader(testString));
       reader.parse(input);
   }
}
