/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.web.tomcat.security;

import java.io.IOException;
import javax.servlet.ServletException;

import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.Response;
import org.apache.catalina.valves.ValveBase;
import org.jboss.logging.Logger;

/** A valve that provides information on the jaas login exception seen in the
 SecurityAssociation exception data. The useExceptionAsMsg flag indicates if
 the exception message should be set as the http response message. The
 exceptionHeader attribute if set is the header name that should be populated
 with the exception message.
   
 @author Scott.Stark@jboss.org
 @version $Revision: 1.1 $
 */
public class BasicAuthValve
   extends ValveBase
{
   private static Logger log = Logger.getLogger(BasicAuthValve.class);
   private static boolean trace = log.isTraceEnabled();

   private boolean useExceptionAsMsg = false;
   private String exceptionHeader = null;

   public boolean isUseExceptionAsMsg()
   {
      return useExceptionAsMsg;
   }
   public void setUseExceptionAsMsg(boolean useExceptionAsMsg)
   {
      this.useExceptionAsMsg = useExceptionAsMsg;
   }

   public String getExceptionHeader()
   {
      return exceptionHeader;
   }
   public void setExceptionHeader(String exceptionHeader)
   {
      this.exceptionHeader = exceptionHeader;
   }

   public void invoke(Request request, Response response)
      throws IOException, ServletException
   {
      getNext().invoke(request, response);
      // Check the SecurityAssociation context exception
      Throwable t = (Throwable) SecurityAssociationActions.getAuthException();
      int status = response.getStatus();
      
      if( trace )
         log.trace("Status: "+status+"SecurityAssociation.exception: ", t);
      if( status >= 400 && t != null )
      {
         String msg = t.getMessage();
         // Set the response msg
         if( useExceptionAsMsg )
         {
            if( response.getCoyoteResponse() != null )
               response.getCoyoteResponse().setMessage(msg);
         }
         // Set the response exception header
         if( exceptionHeader != null )
            response.setHeader(exceptionHeader, msg);
      }
   }
}
