/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.tm;

import javax.transaction.InvalidTransactionException;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

import org.jboss.util.NestedRuntimeException;

/**
 * A simple helper for transaction demarcation.
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.1.2.1 $
 */
public class TransactionDemarcationSupport
{
   private static final TransactionManagerLocator locator = TransactionManagerLocator.getInstance();
   
   /**
    * Suspend any transaction associated with the thread
    * 
    * @return any suspended transaction or null if there was no transaction
    */
   public static Transaction suspendAnyTransaction()
   {
      TransactionManager tm = locator.locate();
      try
      {
         return tm.suspend();
      }
      catch (SystemException e)
      {
         throw new NestedRuntimeException("Unable to suspend transaction", e);
      }
   }
   
   /**
    * Resume any transaction
    * 
    * @param tx the transaction to resume
    */
   public static void resumeAnyTransaction(Transaction tx)
   {
      // Nothing to do
      if (tx == null)
         return;
      
      TransactionManager tm = locator.locate();
      try
      {
         tm.resume(tx);
      }
      catch (InvalidTransactionException e)
      {
         throw new NestedRuntimeException("Unable to resume an invalid transaction", e);
      }
      catch (SystemException e)
      {
         throw new NestedRuntimeException("Unable to suspend transaction", e);
      }
   }
}
