/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.tm;

import javax.management.ObjectName;
import javax.transaction.TransactionManager;

import org.jboss.mx.util.ObjectNameFactory;
import org.jboss.system.ServiceMBean;
import org.jboss.tm.integrity.TransactionIntegrityFactory;

/**
 * MBean interface.
 * @see TxManager
 */
public interface TransactionManagerServiceMBean extends ServiceMBean
{
   static final ObjectName OBJECT_NAME = ObjectNameFactory.create("jboss:service=TransactionManager");
   
   /**
    * Describe <code>getGlobalIdsEnabled</code> method here.
    * @return an <code>boolean</code> value
    */
   boolean getGlobalIdsEnabled();

   /**
    * Describe <code>setGlobalIdsEnabled</code> method here.
    * @param newValue an <code>boolean</code> value
    */
   void setGlobalIdsEnabled(boolean newValue);

   /**
    * Is thread interruption enabled at transaction timeout
    * @return true for interrupt threads, false otherwise    */
   boolean isInterruptThreads();

   /**
    * Enable/disable thread interruption at transaction timeout.
    * @param interruptThreads pass true to interrupt threads, false otherwise    */
   void setInterruptThreads(boolean interruptThreads);

   /**
    * Describe <code>getTransactionTimeout</code> method here.
    * @return an <code>int</code> value
    */
   int getTransactionTimeout();

   /**
    * Describe <code>setTransactionTimeout</code> method here.
    * @param timeout an <code>int</code> value
    */
   void setTransactionTimeout(int timeout);

   /**
    * mbean get-set pair for field xidFactory Get the value of xidFactory
    * @return value of xidFactory
    */
   ObjectName getXidFactory();

   /**
    * Set the value of xidFactory
    * @param xidFactory Value to assign to xidFactory
    */
   void setXidFactory(ObjectName xidFactory);

   /**
    * mbean get-set pair for field transactionManager Get the value of transactionManager
    * @return value of transactionManager
    */
   TransactionManager getTransactionManager();

   /**
    * Get the xa terminator
    * @return the xa terminator    */
   JBossXATerminator getXATerminator();

   /**
    * Counts the number of transactions
    * @return the number of active transactions
    */
   long getTransactionCount();

   /**
    * The number of commits.
    * @return the number of transactions that have been committed
    */
   long getCommitCount();

   /**
    * The number of rollbacks.
    * @return the number of transactions that have been rolled back
    */
   long getRollbackCount();

   /**
    * The <code>registerXAExceptionFormatter</code> method
    * @param clazz a <code>Class</code> value
    * @param formatter a <code>XAExceptionFormatter</code> value
    */
   void registerXAExceptionFormatter(Class clazz, XAExceptionFormatter formatter);

   /**
    * The <code>unregisterXAExceptionFormatter</code> method
    * @param clazz a <code>Class</code> value
    */
   void unregisterXAExceptionFormatter(Class clazz);
   
   /**
    * Set the Integrity checker factory 
    * 
    * @param factory the integrity checker factory
    */
   void setTransactionIntegrityFactory(TransactionIntegrityFactory factory);

}
