/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.monitor.alarm;

import javax.management.ObjectName;

/**
 * AlarmKey
 *
 * Used to correlate stateful alarms based on mbean+type match.
 *
 * @author  <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.4.3 $
 */
class AlarmKey
{
   // Private Data --------------------------------------------------
   
   private String alarmId;

   // Static Helper -------------------------------------------------  
   
   public static AlarmKey createKey(Object mbean, String type)
   {
      if (mbean instanceof ObjectName)
      {
         return new AlarmKey(mbean.toString(), type);
      }
      else if (mbean instanceof String)
      {
         return new AlarmKey((String)mbean, type);
      }
      else
      {
         throw new IllegalArgumentException("Bad non-ObjectName 'mbean' parameter: " + mbean);
      }
   }
   
   // Constructor ---------------------------------------------------
   
   public AlarmKey(String mbean, String type)
   {
      StringBuffer sbuf = new StringBuffer(256);
      
      sbuf.append(mbean).append('+').append(type);
      
      alarmId = sbuf.toString();
   }
   
   // Object stuff --------------------------------------------------
   
   /**
    * Bases equality on alarmId equality
    */
   public boolean equals(Object obj)
   {
      if (obj instanceof AlarmKey)
      {
         return ((AlarmKey)obj).alarmId.equals(this.alarmId);
      }
      else
      {
         return false;
      }
   }
   
   /**
    * Makes use of alarmId's hashCode
    */
   public int hashCode()
   {
      return this.alarmId.hashCode();
   }
   
   /**
    * Pretty prints
    */
   public String toString()
   {
      StringBuffer sbuf = new StringBuffer(256);
      
      sbuf.append("AlarmKey[").append(this.alarmId).append("]");
      
      return sbuf.toString();
   }
}
