/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.monitor.services;

import javax.management.ObjectName;

import org.jboss.monitor.alarm.AlarmTableNotification;
import org.jboss.mx.util.ObjectNameFactory;
import org.jboss.system.ListenerServiceMBean;

/**
 * MBean interface.
 * 
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.4.5 $
 */
public interface ActiveAlarmTableMBean extends ListenerServiceMBean
{
   ObjectName OBJECT_NAME = ObjectNameFactory.create("jboss.monitor:service=ActiveAlarmTable");
   
   // Attributes ----------------------------------------------------
   
   /**
    * The unique serverId
    */
   void setServerId(String serverId);
   String getServerId();

   /**
    * The dynamic log level
    */
   void setLogLevel(String logLevel);
   String getLogLevel();
   
   /**
    * The maximum number of alarms to keep, use -1 to disable
    */
   void setMaxTableSize(int maxSize);
   int getMaxTableSize();
   
   /**
    * Number of notifications received.
    */
   long getNotificationsReceived();

   /**
    * Number of active alarms in the table
    */
   int getActiveAlarmCount();
   
   // Operations ----------------------------------------------------
   
   /**
    * Acknowledge all alarms
    * @return number of acknowledged alarms
    */
   int acknowledgeAll(String user, String system);

   /**
    * Uncknowledge all alarms
    * @return number of unacknowledged alarms
    */
   int unacknowledgeAll(String user, String system);

   /**
    * Acknowledge an Alarm
    * @return true if ack was succesful, false otherwise (not in table or acked already)
    */
   boolean acknowledge(String alarmId, String user, String system);

   /**
    * Unacknowledge an Alarm
    * @return true if unack was succesful, false otherwise (not in table or unacked already)
    */
   boolean unacknowledge(String alarmId, String user, String system);

   /**
    * Gets the ActiveAlarmTable
    */
   AlarmTableNotification[] fetchAlarmTable();

   /**
    * Gets the ActiveAlarmTable as Html
    */
   String fetchAlarmTableAsHtml();

}
