/*  
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.services.binding;

import java.net.URL;

import javax.management.ObjectName;

import org.jboss.mx.util.ObjectNameFactory;

/**
 * ServiceBindingManager MBean interface
 * 
 * @author <a href="mailto:bitpushr@rochester.rr.com">Mike Finn</a>
 * @author Scott.Stark@jboss.org
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.4.3 $
 */
public interface ServiceBindingManagerMBean extends org.jboss.system.ServiceBinding
{
   /** Default ObjectName */
   static final ObjectName OBJECT_NAME = ObjectNameFactory.create("jboss.system:service=ServiceBindingManager");
   
   // Attributes ----------------------------------------------------

   /**
    * The name of the server this manager is associated with. This is a
    * logical name used to lookup ServiceConfigs from the ServicesStore.
    */   
   void setServerName(String serverName);
   String getServerName();

   /**
    * The name of the class implementation of the ServicesStoreFatory. The
    * default value is org.jboss.services.binding.XMLServicesStoreFactory.
    */
   void setStoreFactoryClassName(String storeFactoryClassName);
   String getStoreFactoryClassName();
   
   /**
    * The URL of the configuration store.
    */
   void setStoreURL(URL storeURL);   
   URL getStoreURL();

   /**
    * The ObjectName of the ServiceController,
    * defaults to ServiceControllerMBean.OBJECT_NAME
    */
   void setServiceController(ObjectName serviceController) throws Exception;
   ObjectName getServiceController();
   
   // Operations ----------------------------------------------------

   /**
    * Looks up the service config for the given service,
    * using the server name bound to this mbean.
    * 
    * @param serviceName the JMX ObjectName of the service
    * @return ServiceConfig instance if found, null otherwise 
    */
   ServiceConfig getServiceConfig(ObjectName serviceName) throws Exception;

}
