/*  
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.services.loggingmonitor;

import java.util.HashMap;
import java.util.Map;

/**
 * This class encapsulates the specification of a log file's rollover period.
 * 
 * @author <a href="mailto:jimmy.wilson@acxiom.com">James Wilson</a>
 * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis</a>
 * @version $Revision: 1.1.2.2 $
 */
class RolloverPeriod
{
   private static final Map periodFormatsMap;
   static
   {
      periodFormatsMap = new HashMap();
      periodFormatsMap.put("MONTH",     "\'.\'yyyy-MM");
      periodFormatsMap.put("MONTHLY",   "\'.\'yyyy-MM");
      periodFormatsMap.put("WEEK",      "\'.\'yyyy-ww");
      periodFormatsMap.put("WEEKLY",    "\'.\'yyyy-ww");
      periodFormatsMap.put("DAY",       "\'.\'yyyy-MM-dd");
      periodFormatsMap.put("DAILY",     "\'.\'yyyy-MM-dd");
      periodFormatsMap.put("HALFDAY",   "\'.\'yyyy-MM-dd-a");
      periodFormatsMap.put("HALFDAILY", "\'.\'yyyy-MM-dd-a");
      periodFormatsMap.put("HOUR",      "\'.\'yyyy-MM-dd-HH");
      periodFormatsMap.put("HOURLY",    "\'.\'yyyy-MM-dd-HH");
      periodFormatsMap.put("MINUTE",    "\'.\'yyyy-MM-dd-HH-mm");
   }

   private String rolloverPeriod;
   private String rolloverFormat;

   /**
    * Constructor.
    *
    * @param rolloverPeriod a rollover period specification.
    */
   public RolloverPeriod(String rolloverPeriod)
   {
      this.rolloverFormat = (String)periodFormatsMap.get(rolloverPeriod.toUpperCase());

      if (this.rolloverFormat == null)
      {
         throw new IllegalArgumentException("Unknown rollover period: " + rolloverPeriod);
      }
      this.rolloverPeriod = rolloverPeriod;
   }

   /**
    * Returns the rollover format specification associated with the rollover
    * period specification used to construct this class.
    */
   public String getRolloverFormat()
   {
      return rolloverFormat;
   }

   /**
    * Returns the rollover period specification used to construct this class.
    */
   public String toString()
   {
      return rolloverPeriod;
   }
}
